/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.cmp;


import java.applet.Applet;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.HierarchyEvent;
import java.awt.event.HierarchyListener;

import javax.swing.JDialog;
import javax.swing.JOptionPane;

import net.sf.sfac.setting.Settings;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Dialog storing its bounds (size & position) in settings. <br>
 * This dialog can be of two 'location type':
 * <ul>
 * <li>ABSOLUTE: then it stores and restore its absolute position on screen.
 * <li>RELATIVE: (the default) then it stores and restore its position relatively to it's parent window.
 * </ul>
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class SizedDialog extends JDialog {

    public enum LocationType {
        ABSOLUTE, RELATIVE
    }

    private static Log log = LogFactory.getLog(SizedDialog.class);

    /** Key used to store dialog bounds in settings. */
    private static final String DIALOG_BOUNDS_KEY = ".bounds";
    /** Default bounds of the dialog. */
    private static final Rectangle STATIC_DEFAULT_BOUNDS = new Rectangle(20, 20, 850, 650);

    /** Current settings holder (can be null). */
    private Settings setts;
    private String dialogId;
    /** Note: if locType==RELATIVE the x,y part of the bounds is used as a shift added to the parent window location. */
    private Rectangle defaultBounds;
    private LocationType locType = LocationType.RELATIVE;


    /**
     * Construct a sized dialog. If the given settings holder is not null, this dialog will store the dialog bounds.
     * 
     * @param owner
     *            The component triggering this dialog (allowing to find the main frame).
     * @param sett
     *            settings holder object (can be null if this dialog does't have to store settings).
     * @param dialogBaseId
     *            dialog id used find settings (bounds, ...).
     */
    public SizedDialog(Component owner, Settings sett, String dialogBaseId) {
        super(JOptionPane.getFrameForComponent(owner));
        dialogId = dialogBaseId;
        init(sett);
    }


    /**
     * Construct an sized dialog. If the given settings holder is not null, this dialog will store the dialog bounds.
     * 
     * @param owner
     *            The dialog triggering this dialog (allowing to find the main frame).
     * @param sett
     *            settings holder object (can be null if this dialog does't have to store settings).
     * @param dialogBaseId
     *            dialog id used find settings (bounds, ...).
     */
    public SizedDialog(Dialog owner, Settings sett, String dialogBaseId) {
        super(owner);
        dialogId = dialogBaseId;
        init(sett);
    }


    public Settings getSettings() {
        return setts;
    }


    public LocationType getLocationType() {
        return locType;
    }


    public void setLocationType(LocationType newLocType) {
        locType = newLocType;
    }


    public Rectangle getDefaultBounds() {
        return (defaultBounds == null) ? STATIC_DEFAULT_BOUNDS : defaultBounds;
    }


    public void setDefaultBounds(Rectangle newDefaultBounds) {
        defaultBounds = newDefaultBounds;
    }


    public void setDefaultBounds(LocationType newLocType, Rectangle newDefaultBounds) {
        locType = newLocType;
        defaultBounds = newDefaultBounds;
    }


    /**
     * Set the location of this dialog to the position of the window containing the given component + (spaceX, spaceY).
     * 
     * @param c
     *            Any component on the window that will be the basis to calculate the position (can be the window - JFrame or
     *            JDialog itself). If this window is not showing this method will have no effect.
     * @param spaceX
     *            Horizontal space added to the basis window position
     * @param spaceY
     *            Vertical space added to the basis window position
     */
    public void setLocationRelativeTo(Component c, int spaceX, int spaceY, int proposedWidth, int proposedHeight) {
        Container root = getRootContainer(c);
        int targetX = 10;
        int targetY = 10;
        int sizeX = proposedWidth;
        int sizeY = proposedHeight;
        if ((sizeX < 5) || (sizeY < 5)) {
            pack();
            log.warn("Bad proposed with for dialog " + getKeyPrefix() + " (was " + sizeX + "x" + sizeY + ") --> pack to "
                    + getWidth() + "x" + getHeight());
            sizeX = getWidth();
            sizeY = getHeight();
        }
        if (root != null && root.isShowing()) {
            if (log.isDebugEnabled()) {
                log.debug("Locate dialog " + getKeyPrefix() + " at (" + spaceX + "," + spaceY + ") relative to " + root + " at "
                        + root.getBounds());
            }
            Point parentLocation = root.getLocationOnScreen();
            targetX = parentLocation.x + spaceX;
            targetY = parentLocation.y + spaceY;
            if (targetX < 0) targetX = 0;
            if (targetY < 0) targetY = 0;
            Dimension screenSize = getToolkit().getScreenSize();
            if ((targetX + sizeX > screenSize.width) || (targetY + sizeY > screenSize.height)) {
                log.warn("Unable to locate dialog of size " + sizeX + "x" + sizeY + " at point " + targetX + "," + targetY
                        + " for screen size = " + screenSize + " --> locate at (0,0)");
                targetX = 0;
                targetY = 0;
            }
        } else {
            log.warn("Cannot locate dialog relative to a non-showing component");
        }
        if (log.isDebugEnabled()) {
            log.debug("Locate dialog " + getKeyPrefix() + " at: " + targetX + "," + targetY + " (size = " + sizeX + "x" + sizeY
                    + ")");
        }
        setBounds(targetX, targetY, sizeX, sizeY);
    }


    private Container getRootContainer(Component c) {
        Container root = null;
        if (c != null) {
            if (c instanceof Window || c instanceof Applet) {
                root = (Container) c;
            } else {
                Container parent;
                for (parent = c.getParent(); parent != null; parent = parent.getParent()) {
                    if (parent instanceof Window || parent instanceof Applet) {
                        root = parent;
                        break;
                    }
                }
            }
        }
        return root;
    }


    private void init(Settings sett) {
        setts = sett;
        // register listener to store bounds in settings
        if (setts != null) {
            addComponentListener(new ComponentAdapter() {

                @Override
                public void componentResized(ComponentEvent e) {
                    storeDialogBounds();
                }


                @Override
                public void componentMoved(ComponentEvent e) {
                    storeDialogBounds();
                }
            });
        }
        // add a hierarchy listener to be able to initialize lazily the dialog size.
        addHierarchyListener(new HierarchyListener() {

            public void hierarchyChanged(HierarchyEvent he) {
                long flags = he.getChangeFlags();
                if ((flags & HierarchyEvent.DISPLAYABILITY_CHANGED) != 0) {
                    initDialogBounds();
                }
            }
        });
    }


    /**
     * Prefix for keys used to store settings. <br>
     * Subclasses can override this method to have their own settings.
     * 
     * @return Prefix for keys used to store settings.
     */
    protected String getKeyPrefix() {
        return dialogId;
    }


    /**
     * Set bounds of this dialog and add component listener to be notified of bound changes (so we can store last bounds in
     * settings). This method is called only if the settings holder is not null.
     */
    void initDialogBounds() {
        Rectangle rect = getDefaultBounds();
        if (setts != null) rect = setts.getRectangleProperty(getKeyPrefix() + DIALOG_BOUNDS_KEY, rect);
        if (locType == LocationType.ABSOLUTE) {
            if (log.isDebugEnabled()) log.debug("Set absolute dialog " + getKeyPrefix() + " bounds to: " + rect);
            setBounds(rect);
        } else {
            if (log.isDebugEnabled()) {
                log.debug("Set dialog " + getKeyPrefix() + " size to: " + rect.width + "x" + rect.height + " and position ("
                        + rect.x + "," + rect.y + ") relative to " + getOwner());
            }
            setLocationRelativeTo(getOwner(), rect.x, rect.y, rect.width, rect.height);
        }
    }


    void storeDialogBounds() {
        Rectangle bounds = getBounds();
        if (locType == LocationType.RELATIVE) {
            Container root = getRootContainer(getOwner());
            if (root != null && root.isShowing()) {
                Point rootLocation = root.getLocationOnScreen();
                bounds.x -= rootLocation.x;
                bounds.y -= rootLocation.y;
            } else {
                Rectangle defBounds = getDefaultBounds();
                bounds.setLocation(defBounds.x, defBounds.y);
            }
        }
        setts.setRectangleProperty(getKeyPrefix() + DIALOG_BOUNDS_KEY, bounds);
    }

}
