/*-------------------------------------------------------------------------
 Copyright 2012 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor;


import java.util.ArrayList;
import java.util.List;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import net.sf.sfac.gui.editor.EditorModificationEvent.EditorModificationType;


/**
 * Class designed to help implementation of the firing and propagation of EditorModification events in various ObjectEditor
 * implementation.
 * 
 * @author Olivier Berlanger
 */
public class EditorModificationSupport {


    private ObjectEditor source;
    private boolean enableEvents;
    private List<EditorModificationListener> listeners;
    private EditorModificationListener listenerForSubEditors;
    private ListDataListener listModelListener;
    private TableModelListener tableModelListener;


    public EditorModificationSupport(ObjectEditor newSource) {
        if (newSource == null) throw new IllegalArgumentException("Source cannot be null");
        source = newSource;
        enableEvents = true;
    }


    public void addEditorModificationListener(EditorModificationListener lis) {
        if (lis == null) throw new IllegalArgumentException("Added litener cannot be null");
        if (listeners == null) listeners = new ArrayList<EditorModificationListener>();
        listeners.add(lis);
    }


    public void removeEditorModificationListener(EditorModificationListener lis) {
        if (listeners != null) listeners.remove(lis);
    }


    public void fireEditorValueModified(Object oldEditedValue) {
        if (enableEvents && (listeners != null) && (listeners.size() > 0)) {
            EditorModificationEvent eme = new EditorModificationEvent(source, oldEditedValue);
            for (EditorModificationListener lis : listeners) {
                lis.editorModified(eme);
            }
        }
    }


    public boolean getEnableEvents() {
        return enableEvents;
    }


    public void setEnableEvents(boolean newEnableEvents) {
        enableEvents = newEnableEvents;
    }


    public EditorModificationListener getListenerForSubEditors() {
        if (listenerForSubEditors == null) {
            listenerForSubEditors = new EditorModificationListener() {


                public void editorModified(EditorModificationEvent eme) {
                    fireSubEditorModified(eme);
                }
            };
        }
        return listenerForSubEditors;
    }


    public ListDataListener getListenerForListModel() {
        if (listModelListener == null) {
            listModelListener = new ListDataListener() {


                public void intervalRemoved(ListDataEvent e) {
                    fireEditorValueModified(e);
                }


                public void intervalAdded(ListDataEvent e) {
                    fireEditorValueModified(e);
                }


                public void contentsChanged(ListDataEvent e) {
                    fireEditorValueModified(e);
                }
            };
        }
        return listModelListener;
    }


    public TableModelListener getListenerForTableModel() {
        if (tableModelListener == null) {
            tableModelListener = new TableModelListener() {


                public void tableChanged(TableModelEvent e) {
                    fireEditorValueModified(e);
                }
            };
        }
        return tableModelListener;
    }


    public void fireSubEditorModified(EditorModificationEvent subEme) {
        if (enableEvents && (listeners != null) && (listeners.size() > 0)) {
            EditorModificationEvent eme = new EditorModificationEvent(source, subEme);
            for (EditorModificationListener lis : listeners) {
                lis.editorModified(eme);
            }
        }
    }


    public void fireEditorVisibilityModified(boolean oldVisibility) {
        if (enableEvents && (listeners != null) && (listeners.size() > 0)) {
            EditorModificationEvent eme = new EditorModificationEvent(EditorModificationType.COMPONENT_VISIBILITY, source,
                    Boolean.valueOf(oldVisibility));
            for (EditorModificationListener lis : listeners) {
                lis.editorModified(eme);
            }
        }
    }


    public void fireEditorEditabilityModified() {
        if (enableEvents && (listeners != null) && (listeners.size() > 0)) {
            EditorModificationEvent eme = new EditorModificationEvent(EditorModificationType.COMPONENT_EDITABLE, source, null);
            for (EditorModificationListener lis : listeners) {
                lis.editorModified(eme);
            }
        }
    }

}
