/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor;


import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.Collections;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;

import net.sf.sfac.gui.cmp.SizedDialog;
import net.sf.sfac.lang.LanguageSupport;
import net.sf.sfac.setting.Settings;
import net.sf.sfac.utils.Comparison;


/**
 * A simple input dialog class using a GenericObjectEditor to edit the provided Object.
 * <p>
 * Typical usage (with settings to store size):
 * 
 * <pre>
 * EditorOptionsDialog dial = new EditorOptionsDialog(cmp, getSettings(), &quot;some.id&quot;);
 * MyParams params = new MyParams();
 * Object result = dial.showAndEditObject(params);
 * if (&quot;Ok&quot;.equals(result)) {
 *     // use params values
 * }
 * </pre>
 * 
 * </p>
 * <p>
 * The dialog can also be used without settings, in this case the dialog is packed and displayed relatively to the given parent
 * component.
 * </p>
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class EditorOptionsDialog extends SizedDialog implements EditorContext {


    private Component parent;
    private Object[] availableOptions = { "OK", "CANCEL" };
    private Object defaultOption = "OK";
    private Object choosenOption;
    private ObjectEditor editor;


    public EditorOptionsDialog(Component owner) {
        this(owner, null, "options.dialog");
    }


    public EditorOptionsDialog(Component owner, Settings sett, String dialogBaseId) {
        super(owner, sett, dialogBaseId);
        setModal(true);
        parent = owner;
    }


    /**
     * The given options can be anything. You can use, for example, simple strings or enumerated values. <br>
     * By default, options are {"Ok", "Cancel"}.
     * 
     * @param options
     */
    public void setAvailableOptions(Object[] options) {
        availableOptions = options;
        defaultOption = options[0];
    }


    /**
     * The default option is only used to define the default button in the root pane. <br>
     * By default, the default option is the first option in the available options array. Setting the default option to null
     * will cause that there is no default button.
     * <p>
     * Of course, the default option must be set before the GUI is created.
     * </p>
     */
    public void setDefaultOption(Object newDefaultOption) {
        defaultOption = newDefaultOption;
    }


    public Object showAndEditObject(Object toEdit) {
        choosenOption = null;
        editor = EditorFactory.getInstance().createEditor(toEdit.getClass());
        editor.edit(toEdit);
        buildGui();
        if (getSettings() == null) {
            pack();
            setLocationRelativeTo(parent);
        }
        setVisible(true);
        return choosenOption;
    }


    private void buildGui() {
        JPanel contentPane = new JPanel(new BorderLayout());
        setContentPane(contentPane);
        contentPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        contentPane.add(editor.getEditorComponent(), BorderLayout.CENTER);

        JPanel buttonPane = new JPanel();
        buttonPane.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
        int len = availableOptions.length;
        for (int i = 0; i < len; i++) {
            final int optionIndex = i;
            JButton but = new JButton(getLabelForOption(i));
            buttonPane.add(but);
            if (Comparison.equals(defaultOption, availableOptions[i])) {
                getRootPane().setDefaultButton(but);
            }
            but.addActionListener(new ActionListener() {


                public void actionPerformed(ActionEvent e) {
                    closeDialog(optionIndex);
                }
            });
        }
        contentPane.add(buttonPane, BorderLayout.SOUTH);
    }


    private String getLabelForOption(int i) {
        Object option = availableOptions[i];
        String defaultValue = option.toString();
        String key = defaultValue;
        if (option instanceof Enum) {
            key = option.getClass().getSimpleName() + "." + ((Enum<?>) option).name();
        }
        return LanguageSupport.getOptionalLocalizedString(key, defaultValue);
    }


    void closeDialog(int optionIndex) {
        editor.updateModel();
        choosenOption = availableOptions[optionIndex];
        dispose();
    }


    // ------------------------ EditorContext interface implementation ------------------------------------------------


    public boolean canEditInContext(boolean create, Class<?> objectClass, ObjectEditor source) {
        return false;
    }


    public <T> T editInContext(T obj, Class<T> objectClass, ObjectEditor source) {
        return null;
    }


    public void editorModified(boolean modified, ObjectEditor source) {
    }


    public Object getContextObject(String objectKey) {
        return null;
    }


    public EditorFactory getEditorFactory() {
        return EditorFactory.getInstance();
    }


    public Collection<ObjectEditor> getEditors() {
        if (editor == null) return Collections.emptyList();
        return Collections.singleton(editor);
    }


}
