/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor;


import javax.swing.JComponent;


/**
 * Interface of an object editor. <br>
 * The object editor allows to edit objects of a specific class. In a Model-view-controller paradigm the ObjectEditor is the
 * controller, the editedObject is the model and the editorComponent (created by this ObjectEditor) is the view.
 * <p>
 * Concepts:
 * <ul>
 * <li>The edited object class: is the class of the edited object. So, it's the class of the object passed to the edit(Object)
 * method and, hence, the class passed as parameter to the ModelAccess getModelValue(Object) method.
 * <li>The model class: is the class of the object shown in the editor. So, it's the class of the object returned by the
 * getModelValue() (not present in the interface but only in the implementation) and, hence, the class of the object returned by the
 * ModelAccess getModelValue(Object) method.
 * </ul>
 * 
 * @author Olivier Berlanger
 */
public interface ObjectEditor {

    public static final String EDITOR_LABEL_PROPERTY = "label";
    public static final String EDITOR_DESCRIPTION_PROPERTY = "description";
    public static final String EDITOR_INDEX_PROPERTY = "index";
    public static final String EDITOR_TYPE_PROPERTY = "type";
    public static final String EDITOR_LIST_PROPERTY = "list";
    public static final String VISIBILITY_PROPERTY = "visibility";
    public static final String EDITABLE_PROPERTY = "editable";
    public static final String ENABLED_PROPERTY = "enabled";
    public static final String TAB_PROPERTY = "tab";


    /**
     * Edit the given object. <br>
     * Internally, this method implementation should call updateView after setting the new edited object (so GUI displays the newly
     * edited object). <br>
     * Note: the given object can be null, in this case the visual components must be emptyed and disabled.
     * 
     * @param objToEdit
     *            the object to edit.
     */
    public void edit(Object objToEdit);


    /**
     * Get the edited object (the same object instance as the one provided with the <code>edit(..)</code> method). <br>
     * Note: the object is returned "as is", to be sure that it is synchronized with what is displayed (and possibly modified) in
     * the GUI, you must call <code>updateModel()</code> first.
     */
    public Object getEditedObject();


    /**
     * Returns the object used to access editor model.
     */
    public ModelAccess getModelAccess();


    /**
     * Sets the object used to access editor model. <br>
     * The 'model access' should always be set just after the construction of the editor because it provide the editeClass and
     * modelClass which are used as base for many editor configurations.
     */
    public void setModelAccess(ModelAccess newAccess);


    public void setEditorContext(EditorContext context);


    public EditorContext getEditorContext();


    // ----------- Visual components access --------------------------------------------

    /**
     * Get the visual component displaying the editor label.
     */
    public JComponent getLabelComponent();


    /**
     * Get the visual component to be used to edit the object (without the label).
     */
    public JComponent getObjectComponent();


    /**
     * Get the visual component to be used to edit the object. <br>
     * The returned component combines the label (returned by the <code>getLabelComponent()</code> method) and the object editor
     * (returned by the <code>getObjectComponent()</code> method).
     */
    public JComponent getEditorComponent();


    /**
     * Ask the editor to update the visual components with the actual values of the edited object.
     */
    public void updateView();


    /**
     * Ask the editor to update the edited object with the actual values of the visual components. <br>
     * Note: the editor must check that it is editable and enabled before updating the model.
     */
    public void updateModel();


    public boolean isEditable();


    public boolean isEnabled();


    public boolean isVisible();


    // ------------ Properties management ------------------------------------------------

    /**
     * Set properties used to configure this ObjectEditor. <br>
     * Strandard properties supported by all editors are:
     * <ul>
     * <li>label: the name of the editor displayed as title or label (depending on how it composed).If the LanguageSupport is used,
     * the given name is used as bundle key. If omitted, no label is displayed.
     * </ul>
     * Each editor can define its own properties.
     * 
     * @param name
     *            name of the property.
     * @param value
     *            new value of the property. A null value is equivalent to removing the property.
     */
    public void setProperty(String name, Object value);


    /**
     * Convenient method to get a property as an int. This method tries it best to convert the actual object (ex: a String) to an
     * int.
     */
    public int getIntProperty(String name, int defaultValue);


    /**
     * Convenient method to get a property as an double. This method tries it best to convert the actual object (ex: a String) to an
     * double.
     */
    public double getDoubleProperty(String name, double defaultValue);


    /**
     * Get a property of this object editor.
     * 
     * @param name
     *            name of the property.
     * @param value
     *            default value of the property returned if the property is not defined.
     * @return the property value or the default value if the property is not defined.
     */
    public Object getProperty(String name, Object defaultValue);


    // --------------- Events -------------------------------------------------------

    public void addEditorModificationListener(EditorModificationListener lis);


    public void removeEditorModificationListener(EditorModificationListener lis);

    // ---------------- Continuous Validation ------------------------------------------

    // TODO: implement continuous validation

    // public void displayValidationMessages(List<ValidationMessage> messages);

}
