/*-------------------------------------------------------------------------
 Copyright 2012 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor;


import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;

import net.sf.sfac.editor.ValidationMessage.Level;
import net.sf.sfac.editor.ValidationReport;
import net.sf.sfac.editor.ValidationReportImpl;
import net.sf.sfac.editor.Validator;
import net.sf.sfac.editor.ValidatorImpl;
import net.sf.sfac.gui.ExceptionDialog;
import net.sf.sfac.gui.cmp.SizedDialog;
import net.sf.sfac.lang.LanguageSupport;
import net.sf.sfac.setting.Settings;


/**
 * A controller allowing to edit an object in a dialog with "OK" and "CANCEL" buttons. <br>
 * The GUI (the form) used to show/edit the object will be created using the EditorFactory. This controller will ensure the object
 * is edited and track if the "OK" button was hit to close the dialog. The actual action to do with the edited object (like saving
 * it in a database) is the responsibility of the caller.
 * 
 * @see EditorFactory
 * @see CommitableObject
 * @author Olivier Berlanger
 */
public class SimpleObjectEditionController {

    private String controllerId;
    private Settings setts;
    private Object objectToEdit;
    private boolean closedWithOK;
    private Validator validator;
    private ValidationReport continuousValidationReport;

    private SizedDialog editionDialog;
    private ObjectEditor editor;
    private JButton okButton;
    private JButton cancelButton;


    public SimpleObjectEditionController(String id, Settings settngs, Object toEdit) {
        if (id == null) throw new IllegalArgumentException("The given id cannot be null");
        if (settngs == null) throw new IllegalArgumentException("The given Settings cannot be null");
        if (toEdit == null) throw new IllegalArgumentException("The given object to edit cannot be null");
        controllerId = id;
        setts = settngs;
        objectToEdit = toEdit;
        validator = new ValidatorImpl();

    }


    public ObjectEditor getEditor() {
        return editor;
    }


    public void setEditor(ObjectEditor newEditor) {
        editor = newEditor;
    }


    public Validator getValidator() {
        return validator;
    }


    public void setValidator(Validator newValidator) {
        validator = newValidator;
    }


    public void showDialog(Component parent) {
        showDialog(parent, null);
    }


    public void showDialog(Component parent, String title) {
        closedWithOK = false;
        editionDialog = new SizedDialog(parent, setts, "dial." + controllerId);
        editionDialog.setTitle(LanguageSupport.getOptionalLocalizedString("dial_" + controllerId + "_title", title));
        editionDialog.setModal(true);
        buildGui(editionDialog.getContentPane());
        editionDialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        editionDialog.setVisible(true);
    }


    public boolean wasClosedWithOK() {
        return closedWithOK;
    }


    void editionStartingEvent() {
        if ((validator != null) && validator.supportsContinuousValidation()) {
            continuousValidationReport = new ValidationReportImpl();
            validator.initContinuousValidationReport(editor.getEditedObject(), continuousValidationReport);
        }
    }


    void editorModifiedEvent(EditorModificationEvent eme) {
        if ((validator != null) && validator.supportsContinuousValidation()) {
            validator.validateModification(editor.getEditedObject(), eme.getValuePath(), eme.getOldValue(), eme.getNewValue(),
                    continuousValidationReport);
        }
    }


    boolean validateBeforeCommit() {
        boolean continueCommit = true;
        if (validator != null) {
            ValidationReport commitReport = new ValidationReportImpl();
            validator.validateBeforeCommit(editor.getEditedObject(), commitReport);
            if (commitReport.getGlobalValidationLevel().greaterThan(Level.OK)) {
                boolean aborted = ValidationDialog.showReportDialog(commitReport, okButton);
                continueCommit = !aborted;
            }
        }
        return continueCommit;
    }


    private void buildGui(Container contentPane) {
        if (editor == null) editor = EditorFactory.getInstance().createEditor(objectToEdit.getClass());
        editor.setEditorContext(new EditorContextImpl(setts, editor));
        editor.edit(objectToEdit);
        editor.addEditorModificationListener(new EditorModificationListener() {

            public void editorModified(EditorModificationEvent eme) {
                editorModifiedEvent(eme);
            }
        });
        contentPane.add(editor.getEditorComponent(), BorderLayout.CENTER);
        JPanel buttonPane = new JPanel();
        okButton = new JButton(LanguageSupport.getOptionalLocalizedString("OK", "Ok"));
        okButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent newE) {
                close(true);
            }

        });
        editionDialog.getRootPane().setDefaultButton(okButton);
        buttonPane.add(okButton);
        cancelButton = new JButton(LanguageSupport.getOptionalLocalizedString("CANCEL", "Cancel"));
        cancelButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent newE) {
                close(false);
            }

        });
        buttonPane.add(cancelButton);
        contentPane.add(buttonPane, BorderLayout.SOUTH);
        editionStartingEvent();
    }


    void close(boolean withOk) {
        try {
            boolean continueClose = true;
            if (withOk) {
                continueClose = validateBeforeCommit();
            }
            if (continueClose) {
                closedWithOK = withOk;
                editionDialog.dispose();
            }
        } catch (Exception e) {
            ExceptionDialog.showExceptionDialog(editionDialog, "edition_error", "unexpected_exception", e);
        }
    }

}
