/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor;


import java.applet.Applet;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import net.sf.sfac.editor.ValidationMessage;
import net.sf.sfac.editor.ValidationMessage.Level;
import net.sf.sfac.editor.ValidationReport;
import net.sf.sfac.gui.framework.SharedResources;
import net.sf.sfac.lang.LanguageSupport;
import net.sf.sfac.string.StringUtils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Dialog displaying a ValidationReport.
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class ValidationDialog extends JDialog {

    private static Log log = LogFactory.getLog(ValidationDialog.class);

    private ValidationReport report;

    private boolean aborted;


    public static boolean showReportDialog(ValidationReport report, Component parent) {
        ValidationDialog dial = new ValidationDialog(parent, report);
        dial.setVisible(true);
        return dial.isAborted();
    }


    public ValidationDialog(Component parent, ValidationReport theReport) {
        super(JOptionPane.getFrameForComponent(parent));
        setModal(true);
        report = theReport;
        setTitle();
        buildGui();
        aborted = report.getGlobalValidationLevel() == Level.ERROR;
        setBounds(parent);
    }


    private void setBounds(Component c) {
        pack();
        Container root = null;
        if (c != null) {
            if (c instanceof Window || c instanceof Applet) {
                root = (Container) c;
            } else {
                Container parent;
                for (parent = c.getParent(); parent != null; parent = parent.getParent()) {
                    if (parent instanceof Window || parent instanceof Applet) {
                        root = parent;
                        break;
                    }
                }
            }
        }
        if (root != null && root.isShowing()) {
            Point rootLocation = root.getLocationOnScreen();
            Dimension rootSize = root.getSize();
            int targetX = rootLocation.x + (rootSize.width - getWidth()) / 2;
            int targetY = rootLocation.y + (rootSize.height - getHeight()) / 2;
            setLocation(targetX, targetY);
        } else {
            log.warn("Cannot setBounds relative to a non-showing component");
        }
    }


    private void buildGui() {
        JPanel pane = new JPanel(new GridBagLayout());
        pane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        List<ValidationMessage> validationMessages = report.getValidationMessages();
        int len = validationMessages.size();
        int i;
        for (i = 0; i < len; i++) {
            ValidationMessage msg = validationMessages.get(i);
            JLabel label = new JLabel(msg.getText(), getIcon(msg), JLabel.CENTER);
            pane.add(label, new GridBagConstraints(0, i, 2, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE,
                    new Insets(2, 0, 0, 0), 0, 0));
        }
        setContentPane(pane);
        if (report.getGlobalValidationLevel() == Level.ERROR) {
            JButton okButton = new JButton(LanguageSupport.getOptionalLocalizedString("OK", "Ok"));
            okButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent newE) {
                    abort();
                }

            });
            pane.add(okButton, new GridBagConstraints(0, i, 2, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE,
                    new Insets(8, 0, 0, 0), 0, 0));
        } else {
            JButton continueButton = new JButton(LanguageSupport.getOptionalLocalizedString("CONTINUE", "Continue"));
            continueButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent newE) {
                    dispose();
                }

            });
            pane.add(continueButton, new GridBagConstraints(0, i, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                    new Insets(8, 0, 0, 4), 0, 0));
            JButton abortButton = new JButton(LanguageSupport.getOptionalLocalizedString("ABORT", "Abort"));
            abortButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent newE) {
                    abort();
                }

            });
            pane.add(abortButton, new GridBagConstraints(1, i, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE,
                    new Insets(8, 4, 0, 0), 0, 0));
        }
        setContentPane(pane);
    }


    void abort() {
        aborted = true;
        dispose();
    }


    public boolean isAborted() {
        return aborted;
    }


    private void setTitle() {
        String titleKey;
        switch (report.getGlobalValidationLevel()) {
            case ERROR:
                titleKey = "VALIDATION_ERROR";
                break;
            case WARNING:
                titleKey = "VALIDATION_WARNING";
                break;
            case INFO:
                titleKey = "VALIDATION_INFO";
                break;
            default:
                throw new IllegalStateException("Cannot show valiadation dialog for level: " + report.getGlobalValidationLevel());
        }
        String defaultTitle = StringUtils.firstOfWordsUpperCase(titleKey.replace('_', ' '), true);
        setTitle(LanguageSupport.getOptionalLocalizedString(titleKey, defaultTitle));
    }


    private Icon getIcon(ValidationMessage msg) {
        switch (msg.getLevel()) {
            case ERROR:
                return SharedResources.getIcon("error.gif");
            case WARNING:
                return SharedResources.getIcon("warning.gif");
            case INFO:
                return SharedResources.getIcon("info.gif");
            default:
                throw new IllegalStateException("Cannot show valiadation dialog for level: " + report.getGlobalValidationLevel());
        }
    }

}
