/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.access;


import java.lang.reflect.Method;

import net.sf.sfac.utils.ReflectionUtils;


/**
 * Implementation of the ModelAccess interface where the model value is a bean property. <br>
 * The model value is accessed using reflexion with standard bean getter/setter methods.
 * 
 * @author Olivier Berlanger
 */
public class ReflexionModelAccess extends ChainableModelAccess {


    private String path;
    private Method getter;
    private Method setter;


    public ReflexionModelAccess(String fieldName, Class<?> enclosingClass) {
        this(fieldName, enclosingClass, null);
    }


    public ReflexionModelAccess(String fieldName, Class<?> enclosingClass, Class<?> fieldClass) {
        int lastDotIndex = fieldName.lastIndexOf('.');
        if (lastDotIndex > 0) {
            ReflexionModelAccess parent = new ReflexionModelAccess(fieldName.substring(0, lastDotIndex), enclosingClass);
            setParent(parent);
            fieldName = fieldName.substring(lastDotIndex + 1);
            enclosingClass = parent.getModelClass();
        }
        path = fieldName;
        getter = ReflectionUtils.getGetter(enclosingClass, fieldName, fieldClass);
        setupSetter(fieldName, enclosingClass);
    }


    public ReflexionModelAccess(Class<?> enclosingClass, Method fieldGetter) {
        if (fieldGetter == null) throw new IllegalArgumentException("Getter cannot be null");
        getter = fieldGetter;
        path = ReflectionUtils.getFieldName(fieldGetter);
        setupSetter(path, enclosingClass);
    }


    private void setupSetter(String attributeName, Class<?> enclosingClass) {
        setter = ReflectionUtils.getSetter(enclosingClass, attributeName, getter.getReturnType(), false);
    }


    @Override
    public Object getModelValue(Object baseObject) {
        Object editedObject = super.getModelValue(baseObject);
        if (editedObject == null) return null;
        try {
            Object value = getter.invoke(editedObject, (Object[]) null);
            return value;
        } catch (Exception e) {
            throw new IllegalStateException("Unable to invoke getter " + getter + " on object " + editedObject, e);
        }
    }


    public void setModelValue(Object newModelValue, Object baseObject) {
        Object editedObject = super.getModelValue(baseObject);
        if (editedObject == null) {
            throw new NullPointerException("Unable to set value on null object");
        }
        if (setter == null)
            throw new IllegalStateException("Setter is null cannot set value '" + newModelValue + "' on object " + editedObject);
        try {
            setter.invoke(editedObject, newModelValue);
        } catch (Exception e) {
            throw new IllegalStateException("Unable to invoke setter " + setter + " with value '" + newModelValue + "' on object "
                    + editedObject, e);
        }
    }


    public boolean canUpdateModel() {
        return (setter != null);
    }


    @Override
    public Class<?> getThisEditedObjectClass() {
        return getter.getDeclaringClass();
    }


    public Class<?> getModelClass() {
        return getter.getReturnType();
    }


    @Override
    public String getThisFieldPath() {
        return path;
    }


}
