/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.util.ArrayList;
import java.util.List;

import javax.swing.ListModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;


/**
 * An implementation of ComboBoxEditorModel based on a list of object of a ListModel.
 * 
 * @author Olivier Berlanger
 */
public class ComboBoxEditorModelImpl implements ComboBoxEditorModel {


    private ListModel sourceModel;
    private Object selected;
    private List<ListDataListener> listeners;


    public ComboBoxEditorModelImpl(ListModel model) {
        sourceModel = model;
        listeners = new ArrayList<ListDataListener>();
        model.addListDataListener(new ListDataListener() {


            public void intervalRemoved(ListDataEvent ev) {
                fireIntervaleRemoved(ev.getIndex0(), ev.getIndex1());
            }


            public void intervalAdded(ListDataEvent ev) {
                fireIntervaleAdded(ev.getIndex0(), ev.getIndex1());
            }


            public void contentsChanged(ListDataEvent ev) {
                fireContentsChanged(ev.getIndex0(), ev.getIndex1());
            }
        });
    }


    public ComboBoxEditorModelImpl(List<?> model) {
        sourceModel = new WrappingListModel(model);
        listeners = new ArrayList<ListDataListener>();
    }


    public void fireIntervaleRemoved(int index0, int index1) {
        if (listeners.size() > 0) {
            ListDataEvent ev = new ListDataEvent(this, ListDataEvent.INTERVAL_REMOVED, index0, index1);
            for (ListDataListener lis : listeners) {
                lis.intervalRemoved(ev);
            }
        }
    }


    public void fireIntervaleAdded(int index0, int index1) {
        if (listeners.size() > 0) {
            ListDataEvent ev = new ListDataEvent(this, ListDataEvent.INTERVAL_ADDED, index0, index1);
            for (ListDataListener lis : listeners) {
                lis.intervalAdded(ev);
            }
        }
    }


    public void fireContentsChanged(int index0, int index1) {
        if (listeners.size() > 0) {
            ListDataEvent ev = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, index0, index1);
            for (ListDataListener lis : listeners) {
                lis.contentsChanged(ev);
            }
        }
    }


    /**
     * Ask to fire an event telling that all the data of the comboBox has changed. This is used to force a complete refresh of
     * the GUI (in updateView).
     */
    public void fireAllDataChanged() {
        fireContentsChanged(0, getSize());
    }


    public void fireSelectionChanged() {
        fireContentsChanged(-1, -1);
    }


    public Object getSelectedItem() {
        return selected;
    }


    public void setSelectedItem(Object item) {
        if (selected != item) {
            selected = item;
            fireSelectionChanged();
        }
    }


    public Object getElementAt(int index) {
        return sourceModel.getElementAt(index);
    }


    public int getSize() {
        return sourceModel.getSize();
    }


    public void addListDataListener(ListDataListener lis) {
        listeners.add(lis);
    }


    public void removeListDataListener(ListDataListener lis) {
        listeners.remove(lis);
    }


    /**
     * A listModel simply wrapping a list but generating no events.
     */
    static class WrappingListModel implements ListModel {


        private List<?> innerList;


        WrappingListModel(List<?> lst) {
            innerList = lst;
        }


        public Object getElementAt(int index) {
            return innerList.get(index);
        }


        public int getSize() {
            return (innerList == null) ? 0 : innerList.size();
        }


        public void addListDataListener(ListDataListener lis) {
        }


        public void removeListDataListener(ListDataListener lis) {
        }

    }
}
