/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.sfac.gui.cmp.DateTimeChooser;
import net.sf.sfac.lang.LanguageSupport;
import net.sf.sfac.utils.Comparison;


/**
 * Editor used to edit a date value. <br>
 * The date can be of type 'date', 'dateTime', 'time' and is displayed formatted accordingly in a TextField.
 * 
 * @author Olivier Berlanger
 */
public class DateEditor extends SimpleObjectEditor {

    public static enum DateType {
        DATE, DATE_TIME, TIME
    }

    private DateType type;
    private DateFormat format;

    private JComponent component;
    private JTextField dateField;
    private JButton browseButton;
    private DateTimeChooser chooser;


    public DateEditor() {
        this(DateType.DATE);
    }


    public DateEditor(DateType typeOfDate) {
        setTypeOfDate(typeOfDate);
    }


    public void setTypeOfDate(DateType typeOfDate) {
        if (type != typeOfDate) {
            type = typeOfDate;
            switch (type) {
                case DATE_TIME:
                    format = DateFormat.getDateTimeInstance();
                    break;
                case TIME:
                    format = DateFormat.getTimeInstance();
                    break;
                case DATE:
                    format = DateFormat.getDateInstance();
                    break;
            }
        }
    }


    public DateType getTypeOfDate() {
        return type;
    }


    public JComponent getObjectComponent() {
        if (component == null) {
            dateField = new JTextField(25);
            registerFocusListener(dateField);
            // create browse button
            if (type != DateType.TIME) {
                Dimension tfSize = dateField.getPreferredSize();
                browseButton = new JButton("...");
                browseButton.setPreferredSize(new Dimension(tfSize.height, tfSize.height));
                browseButton.addActionListener(new ActionListener() {

                    public void actionPerformed(ActionEvent e) {
                        chooseDate();
                    }
                });
                String tooltip = LanguageSupport.getOptionalLocalizedString("OPEN_DATE_CHOOSER", "Open date chooser");
                browseButton.setToolTipText(tooltip);
                // Put them in a JPanel
                component = new JPanel(new BorderLayout());
                component.add(dateField, BorderLayout.CENTER);
                component.add(browseButton, BorderLayout.EAST);
            } else {
                component = dateField;
            }
            synchronizeEditableState();
            updateView();
        }
        return component;
    }


    void chooseDate() {
        Date initialDate = (Date) getViewValue();
        DateTimeChooser dtChooser = getChooser();
        dtChooser.setLocationRelativeTo(browseButton);
        Date editedDate = dtChooser.editDate(initialDate);
        setViewValue(editedDate);
        updateModel();
    }


    private DateTimeChooser getChooser() {
        if ((chooser == null) && (browseButton != null)) {
            Frame f = JOptionPane.getFrameForComponent(browseButton);
            String title = LanguageSupport.getOptionalLocalizedString("CHOOSE_DATE", "Choose date");
            chooser = new DateTimeChooser(f, title, type == DateType.DATE_TIME);
        }
        return chooser;
    }


    @Override
    protected void synchronizeEditableState() {
        if (dateField != null) {
            dateField.setEditable(isEditable());
            dateField.setEnabled(isEnabled());
        }
    }


    @Override
    protected boolean isViewReady() {
        return (dateField != null);
    }


    @Override
    protected Object getViewValue() {
        Date d = null;
        String text = dateField.getText();
        if (Comparison.isDefined(text)) {
            try {
                d = format.parse(text);
            } catch (Exception e) {
                d = (Date) getModelValue();
                setViewValue(d);
            }
        }
        return d;
    }


    @Override
    protected void setViewValue(Object modelValue) {
        Date modelDate = (Date) modelValue;
        String strDate = (modelDate == null) ? "" : format.format(modelDate);
        dateField.setText(strDate);
    }


    @Override
    public void setProperty(String name, Object value) {
        if (name.equals("type")) {
            String typeStr = (String) value;
            if (typeStr.equalsIgnoreCase("date")) setTypeOfDate(DateType.DATE);
            else if (typeStr.equalsIgnoreCase("dateTime")) setTypeOfDate(DateType.DATE_TIME);
            else if (typeStr.equalsIgnoreCase("time")) setTypeOfDate(DateType.TIME);
            else throw new IllegalArgumentException("Unknown date type: " + value);
        } else {
            super.setProperty(name, value);
        }
    }


    @Override
    public Object getProperty(String name, Object defaultValue) {
        if (name.equals("type")) {
            if (type == DateType.DATE_TIME) return "dateTime";
            else if (type == DateType.TIME) return "time";
            else return "date";
        } else {
            return super.getProperty(name, defaultValue);
        }
    }


    @Override
    protected boolean valueHasChanged(Object oldValue, Object newValue) {
        if (oldValue == newValue) return false;
        if (oldValue == null) return true;
        if (newValue == null) return true;
        return !Comparison.stringEquals(format.format((Date) oldValue), format.format((Date) newValue));
    }

}
