/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.sfac.file.FileType;
import net.sf.sfac.gui.utils.FileDropListener;
import net.sf.sfac.lang.LanguageSupport;


/**
 * Editor used to edit a File value. <br>
 * The file can represent a file or a directory. The File can be contained in a Map (the key is provided) or as attribute of a class
 * (Class and attribute name are provided to access it trough reflexion).
 * 
 * @author Olivier Berlanger
 */
public class FileEditor extends SimpleObjectEditor {

    /** Type of files selected: one of FILES_ONLY, DIRECTORIES_ONLY, FILES_AND_DIRECTORIES property of JFileChooser. */
    private int fileType;
    private JPanel component;
    private JTextField fileNameField;
    private JButton browseButton;


    public FileEditor() {
        this(JFileChooser.FILES_ONLY);
    }


    public FileEditor(int possibleFileType) {
        fileType = possibleFileType;
    }


    protected int getFileType() {
        return fileType;
    }


    protected void setFileType(int newFileType) {
        fileType = newFileType;
    }


    public JComponent getObjectComponent() {
        if (component == null) {
            fileNameField = new JTextField(14);
            registerFocusListener(fileNameField);
            FileDropListener dropListener = new FileDropListener(new FileDropListener.FileDropTarget() {

                public void filesDropped(File[] files, int dropAction) {
                    if ((files != null) && (files.length > 0)) {
                        fileDropped(files[0]);
                    }
                }
            }, (FileType[]) null);
            dropListener.registerToComponent(fileNameField);
            // create browse button
            Dimension tfSize = fileNameField.getPreferredSize();
            browseButton = new JButton("...");
            browseButton.setPreferredSize(new Dimension(tfSize.height, tfSize.height));
            browseButton.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    browse();
                }
            });
            browseButton.setToolTipText(LanguageSupport.getOptionalLocalizedString("OPEN_FILE_CHOOSER", "Open file chooser"));
            // Put them in a JPanel
            component = new JPanel(new BorderLayout());
            component.add(fileNameField, BorderLayout.CENTER);
            component.add(browseButton, BorderLayout.EAST);
            synchronizeEditableState();
            updateView();
        }
        return component;
    }


    void fileDropped(File f) {
        if ((f != null) && (fileType == JFileChooser.DIRECTORIES_ONLY) && !f.isDirectory()) {
            f = f.getParentFile();
        }
        if ((f != null) && (fileNameField != null)) {
            fileNameField.setText(f.getAbsolutePath());
        }
    }


    void browse() {
        File currentFile = new File(fileNameField.getText());
        JFileChooser chooser = new JFileChooser();
        chooser.setFileSelectionMode(fileType);
        if (currentFile.exists()) {
            chooser.setCurrentDirectory(currentFile.getParentFile());
            chooser.setSelectedFile(currentFile);
        }
        int returnVal = chooser.showDialog(browseButton, LanguageSupport.getOptionalLocalizedString("CHOOSE", "Choose"));
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File f = chooser.getSelectedFile();
            String text = f.getAbsolutePath();
            fileNameField.setText(text);
            updateModel();
        }
    }


    @Override
    protected void synchronizeEditableState() {
        if (fileNameField != null) {
            fileNameField.setEditable(isEditable());
            fileNameField.setEnabled(isEnabled());
            browseButton.setEnabled(isEditable() || isEnabled());
        }
    }


    @Override
    protected boolean isViewReady() {
        return (component != null);
    }


    @Override
    protected Object getViewValue() {
        File fil = null;
        String text = fileNameField.getText();
        if (text != null) {
            text = text.trim();
            if (!text.equals("")) {
                fil = new File(text);
            }
        }
        return fil;
    }


    @Override
    protected void setViewValue(Object modelValue) {
        File fil = (File) modelValue;
        String modelText = (fil == null) ? "" : fil.getAbsolutePath();
        fileNameField.setText(modelText);
    }


    @Override
    public void setProperty(String name, Object value) {
        if (name.equals("type")) {
            String type = (String) value;
            if (type.equalsIgnoreCase("file")) fileType = JFileChooser.FILES_ONLY;
            else if (type.equalsIgnoreCase("directory")) fileType = JFileChooser.DIRECTORIES_ONLY;
            else if (type.equalsIgnoreCase("fileAndDirectory")) fileType = JFileChooser.FILES_AND_DIRECTORIES;
        } else {
            super.setProperty(name, value);
        }
    }


    @Override
    public Object getProperty(String name, Object defaultValue) {
        if (name.equals("type")) {
            if (fileType == JFileChooser.DIRECTORIES_ONLY) return "directory";
            else if (fileType == JFileChooser.FILES_AND_DIRECTORIES) return "fileAndDirectory";
            else return "file";
        } else {
            return super.getProperty(name, defaultValue);
        }
    }

}
