/*-------------------------------------------------------------------------
 Copyright 2006 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractListModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;


/**
 * This class is an editor for java.awt.Font objects.
 */
public class FontEditor extends ComboBoxEditor {

    static final Font DEFAULT_FONT = new JLabel().getFont();
    static final Dimension DEFAULT_SIZE = new Dimension(16, 24);

    private static String[] DEFAULT_FONT_NAMES = { "Dialog-plain-12", "Dialog-plain-16", "Dialog-plain-24", "Serif-plain-12",
            "Serif-plain-16", "Serif-plain-24", "Monospaced-plain-12", "Monospaced-plain-16", "Monospaced-plain-24" };


    public FontEditor() {
        this(null);
    }


    public FontEditor(String[] availableFontNames) {
        super(new FontEditorModel());
        setRenderer(new FontRenderer());
        setAvailableFonts((availableFontNames == null) ? getDefaultFontNames() : availableFontNames);
    }


    protected String[] getDefaultFontNames() {
        return DEFAULT_FONT_NAMES;
    }


    protected void setAvailableFonts(String[] fontNames) {
        List<Font> availableFonts = new ArrayList<Font>();
        for (String fontName : fontNames) {
            availableFonts.add(Font.decode(fontName));
        }
        ((FontEditorModel) getComboBoxModel()).setAvailableFonts(availableFonts);
    }


    @Override
    protected void fireObjectSelectionChanged() {
        super.fireObjectSelectionChanged();
        JComboBox comboBox = getComboBox();
        Font selected = (Font) comboBox.getSelectedItem();
        if (selected == null) selected = DEFAULT_FONT;
        comboBox.setFont(selected);
    }


    /**
     * Enable the 'choose font' button. <br>
     * Note: The aim of the 'open' button of the ComboBoxEditor was different, the original intention is
     * <i>"view details of selected objects"</i> while here it is <i>"open font chooser"</i>. It's the reason why the two methods
     * <code>canViewItem()</code> and <code>openSelectedObject()</code> have strange names in this context.
     */
    @Override
    protected boolean canViewItem() {
        return true;
    }


    /**
     * Show the font chooser. <br>
     * Note: The aim of the 'open' button of the ComboBoxEditor was different, the original intention is
     * <i>"view details of selected objects"</i> while here it is <i>"open font chooser"</i>. It's the reason why the two methods
     * <code>canViewItem()</code> and <code>openSelectedObject()</code> have strange names in this context.
     */
    @Override
    protected void openSelectedObject() {
        // TODO: show font chooser
    }

    @SuppressWarnings("serial")
    static class FontEditorModel extends AbstractListModel implements ComboBoxEditorModel {

        private Font selected;
        private List<Font> availableFonts;


        FontEditorModel() {
        }


        public List<Font> getAvailableFonts() {
            return availableFonts;
        }


        public void setAvailableFonts(List<Font> newAvailableFonts) {
            availableFonts = newAvailableFonts;
            fireAllDataChanged();
        }


        public void fireAllDataChanged() {
            fireContentsChanged(this, 0, getSize());
        }


        public Object getSelectedItem() {
            return selected;
        }


        public void setSelectedItem(Object newSelected) {
            if (selected != newSelected) {
                selected = (Font) newSelected;
                fireContentsChanged(this, -1, -1);
            }
        }


        public Object getElementAt(int index) {
            return availableFonts.get(index);
        }


        public int getSize() {
            return (availableFonts == null) ? 0 : availableFonts.size();
        }

    }

    @SuppressWarnings("serial")
    static class FontRenderer extends DefaultListCellRenderer {

        @Override
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean hasFocus) {
            Font f = (value == null) ? DEFAULT_FONT : (Font) value;
            String text = (value == null) ? "<Default font>" : getFontDisplayName(f);
            super.getListCellRendererComponent(list, text, index, isSelected, hasFocus);
            setOpaque(true);
            Font displayDont = (f.getSize() == 12) ? f : f.deriveFont(12f);
            setFont(displayDont);
            setPreferredSize(DEFAULT_SIZE);
            return this;
        }


        public static String getFontDisplayName(Font f) {
            String displayName = null;
            if (f != null) {
                StringBuilder sb = new StringBuilder();
                sb.append(f.getFamily());
                switch (f.getStyle()) {
                    case Font.BOLD:
                        sb.append(" bold");
                        break;
                    case Font.ITALIC:
                        sb.append(" italic");
                        break;
                    case Font.BOLD | Font.ITALIC:
                        sb.append(" bold & talic");
                        break;
                }
                sb.append(", ");
                sb.append(f.getSize());
                displayName = sb.toString();
            }
            return displayName;
        }
    }

}
