/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.lang.reflect.Method;

import net.sf.sfac.editor.EditorConfig;
import net.sf.sfac.gui.editor.EditorConfigHelper;
import net.sf.sfac.gui.editor.EditorFactory;
import net.sf.sfac.gui.editor.ModelAccess;
import net.sf.sfac.gui.editor.ObjectEditor;
import net.sf.sfac.gui.editor.access.ChainableModelAccess;
import net.sf.sfac.gui.editor.access.IndexedModelAccess;
import net.sf.sfac.gui.editor.access.ReflexionModelAccess;
import net.sf.sfac.string.StringUtils;
import net.sf.sfac.utils.ReflectionUtils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Object editor discovering fields to edit by annotations. <br>
 * This class can be taken as basis to build a custom object editor but if you don't use annotations, it's simpler to use the
 * CompositeObjectEditor class.
 * 
 * @author Olivier Berlanger
 */
public class GenericObjectEditor extends CompositeObjectEditor {

    private static Log log = LogFactory.getLog(GenericObjectEditor.class);


    public GenericObjectEditor() {
    }


    @Override
    public void setModelAccess(ModelAccess newAccess) {
        super.setModelAccess(newAccess);
        setupForEditedClass();
    }


    protected void setupForEditedClass() {
        Class<?> theModelClass = getModelClass();
        // set properties defined in edited class annotations
        EditorConfig editorClassAnnotation = EditorFactory.getClassAnnotation(theModelClass);
        if (editorClassAnnotation == null) {
            setProperty(EDITOR_LABEL_PROPERTY, StringUtils.firstToLowerCase(theModelClass.getSimpleName()));
        } else {
            EditorConfigHelper.setProperties(this, editorClassAnnotation, theModelClass.getSimpleName());
        }
        // add sub-editors defined in method annotations (or use introspection)
        addSubEditors();
    }


    protected void addSubEditors() {
        Class<?> editedClass = getModelClass();
        log.info("Add sub-editors for edited class " + editedClass + " in " + this);
        int nbrAdded = addSubEditorsByAnnotations(editedClass, null);
        if (log.isDebugEnabled()) log.debug(" -> " + nbrAdded + " EditorConfig annotations processed");
        if (nbrAdded == 0) addSubEditorsByIntrospection(editedClass);
    }


    public int addSubEditorsByAnnotations(Class<?> theModelClass, ModelAccess parentAccess) {
        int annotationsFound = 0;
        EditorFactory editorFactory = getEditorFactory();
        Method[] methods = theModelClass.getMethods();
        ObjectEditor subEditor;
        EditorConfig fieldAnnotation;
        int len = methods.length;
        for (int i = 0; i < len; i++) {
            Method getter = methods[i];
            fieldAnnotation = getter.getAnnotation(EditorConfig.class);
            if (fieldAnnotation != null) {
                boolean indexedGetter = IndexedModelAccess.isIndexedGetter(getter);
                Class<?> fieldType = getter.getReturnType();
                if ("embed".equals(fieldAnnotation.type())) {
                    if (indexedGetter) throw new IllegalStateException("Cannot embed an indexed getter");
                    ChainableModelAccess access = new ReflexionModelAccess(theModelClass, getter);
                    access.setParent(parentAccess);
                    addSubEditorsByAnnotations(fieldType, access);
                } else {
                    if (log.isDebugEnabled()) log.debug("Add editor for " + getter + ", indexed = " + indexedGetter);
                    ChainableModelAccess access = indexedGetter ? new IndexedModelAccess(theModelClass, getter)
                            : new ReflexionModelAccess(theModelClass, getter);
                    access.setParent(parentAccess);

                    Class<?> editorClass = EditorFactory.getEditorClass(fieldAnnotation);
                    if (editorClass == null) {
                        Class<?> defaultEditorClass = indexedGetter ? TableObjectEditor.class : EmbededObjectEditor.class;
                        subEditor = editorFactory.createEditor(access, defaultEditorClass);
                    } else {
                        subEditor = EditorFactory.newEditorInstance(access, editorClass);
                    }
                    EditorConfigHelper.setProperties(subEditor, fieldAnnotation, ReflectionUtils.getFieldName(getter));
                    addChildEditor(subEditor);
                    annotationsFound++;
                }
            }
        }
        return annotationsFound;
    }


    protected void addSubEditorsByIntrospection(Class<?> theModelClass) {
        log.info("! Add field editors by instrospection.");
        Method[] methods = theModelClass.getDeclaredMethods();
        int len = methods.length;
        for (int i = 0; i < len; i++) {
            Method getter = methods[i];
            Class<?> returnType = getter.getReturnType();
            String methodName = getter.getName();
            if (methodName.startsWith("get") || (methodName.startsWith("is") && returnType.equals(Boolean.TYPE))) {
                Class<?>[] paramTypes = getter.getParameterTypes();
                if (paramTypes.length == 0) {
                    addSubEditor(ReflectionUtils.getFieldName(getter), getter);
                }
            }
        }
    }


    protected void addSubEditor(String fieldLabel, Method getter) {
        Class<?> fieldType = getter.getReturnType();
        log.info("  -> create sub-editor for field '" + fieldLabel + "' of type " + fieldType.getName());
        ModelAccess access = new ReflexionModelAccess(getModelClass(), getter);
        ObjectEditor editor = getEditorFactory().createEditor(access, EmbededObjectEditor.class);
        editor.setProperty(ObjectEditor.EDITOR_LABEL_PROPERTY, fieldLabel);
        addChildEditor(editor);
    }

}
