/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Collection;
import java.util.Collections;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import net.sf.sfac.gui.editor.EditorContext;
import net.sf.sfac.gui.editor.EditorFactory;
import net.sf.sfac.gui.editor.ModelAccess;
import net.sf.sfac.gui.editor.ObjectEditor;
import net.sf.sfac.gui.editor.access.IndexedModelAccess;
import net.sf.sfac.gui.utils.ObjectSelectionListener;
import net.sf.sfac.gui.utils.ObjectSelector;
import net.sf.sfac.setting.Settings;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Editor showing a list of elements and the detail of the selected one.
 * 
 * @author Olivier Berlanger
 */
public class SetAndDetailsEditor extends BaseObjectEditor implements EditorContext, ObjectSelectionListener {

    private static Log log = LogFactory.getLog(SetAndDetailsEditor.class);

    private static final String LIST_DIVIDER_LOCATION_KEY = "ui.setAndDetailsSplitPane.dividerLocation";

    private String title;
    private Object currentItem;
    private MultiObjectEditor detailsEditor;
    private int nbrLists;
    private ObjectEditor[] selectors;
    private ObjectSelector currentSelector;
    private boolean editable = true;
    private boolean enabled = true;

    private JPanel editorPane;
    private JSplitPane splitPane;
    private JLabel titleLabel;


    /**
     * Build a SetAndDetailsEditor without any ObjectEditor to be used for selection. By default a ListObjectEditor is created as
     * ObjectEditor to be used for selection
     */
    public SetAndDetailsEditor() {
        this(new ObjectEditor[] {});
    }


    /**
     * Build a SetAndDetailsEditor with given ObjectEditor to be used for selection.
     * 
     * @param selectionLists
     *            list of ObjectEditor that are used for selection. Important: those objects must implement both ObjectEditor and
     *            ObjectSelector interfaces.
     */
    public SetAndDetailsEditor(ObjectEditor... selectionLists) {
        selectors = selectionLists;
        nbrLists = (selectors == null) ? 0 : selectors.length;
        detailsEditor = new MultiObjectEditor();
        detailsEditor.setEditorContext(this);
        detailsEditor.addEditorModificationListener(getEditorModificationSupport().getListenerForSubEditors());

        setDefaultProperties();
    }


    public int getSelectorCount() {
        return (selectors == null) ? 0 : selectors.length;
    }


    public ObjectEditor getSelector(int index) {
        return selectors[index];
    }


    @Override
    public void setModelAccess(ModelAccess newAccess) {
        super.setModelAccess(newAccess);
        registerSelectors(selectors);
    }


    /**
     * Set the ObjectEditor to be used for selection.
     * 
     * @param selectionLists
     *            list of ObjectEditor that are used for selection. Important: those objects must implement both ObjectEditor and
     *            ObjectSelector interfaces.
     */
    public void registerSelectors(ObjectEditor... selectionLists) {
        selectors = selectionLists;
        nbrLists = (selectors == null) ? 0 : selectors.length;
        if (nbrLists == 0) {
            if (getModelAccess() instanceof IndexedModelAccess) {
                // create a default object selector
                log.info("Create default ListObjectEditor as selector for " + this);
                registerSelectors(new ListObjectEditor());
            } else {
                throw new IllegalArgumentException("You must define at least one selectionLists");
            }
        } else {
            for (int i = 0; i < nbrLists; i++) {
                try {
                    // TODO: guard with ? if (selectors[i].getModelAccess() == null)
                    selectors[i].setModelAccess(getModelAccess());
                    selectors[i].setEditorContext(this);
                    ((ObjectSelector) selectors[i]).addObjectSelectionListener(this);
                } catch (ClassCastException cce) {
                    throw new IllegalArgumentException("The selectionList editor " + selectors[i].getClass().getName()
                            + " must implement the ObjectSelector interface", cce);
                }
            }
        }
    }


    private void setDefaultProperties() {
        setProperty("weightx", Double.valueOf(1.0));
        setProperty("weighty", Double.valueOf(1.0));
        setProperty("fill", Integer.valueOf(GridBagConstraints.BOTH));
    }


    @Override
    public void edit(Object objToEdit) {
        // at this time all the selectors must be defined
        if (nbrLists == 0) throw new IllegalArgumentException("You must define at least one selectionLists");
        // edit the object in this editor and the selectors
        super.edit(objToEdit);
        for (int i = 0; i < nbrLists; i++) {
            selectors[i].edit(objToEdit);
        }
        updateView();
        // by default select the first item of the first list
        ((ObjectSelector) selectors[0]).selectDefaultItem();
    }


    public JComponent getEditorComponent() {
        if (editorPane == null) {
            // at this time all the selectors must be defined
            if (nbrLists == 0) throw new IllegalArgumentException("You must define at least one selectionLists");
            // create main panel
            editorPane = new JPanel(new BorderLayout());
            editorPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
            // title
            titleLabel = new JLabel();
            Font labelFont = titleLabel.getFont();
            titleLabel.setFont(labelFont.deriveFont(2f * labelFont.getSize2D()));
            editorPane.add(titleLabel, BorderLayout.NORTH);
            // List(s) panel
            JPanel listPane = new JPanel(new GridBagLayout());
            for (int i = 0; i < nbrLists; i++) {
                int top = (i == 0) ? 0 : 5;
                listPane.add(selectors[i].getEditorComponent(), new GridBagConstraints(0, i, 1, 1, 1.0, 1.0,
                        GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(top, 0, 0, 0), 0, 0));
            }
            // card pane
            JComponent detailsPane = detailsEditor.getEditorComponent();
            splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, listPane, detailsPane);
            Settings sett = getSettings();
            int dividerLocation = (sett == null) ? 200 : sett.getIntProperty(LIST_DIVIDER_LOCATION_KEY, 200);
            splitPane.setDividerLocation(dividerLocation);
            splitPane.addPropertyChangeListener(JSplitPane.DIVIDER_LOCATION_PROPERTY, new PropertyChangeListener() {

                public void propertyChange(PropertyChangeEvent evt) {
                    storeSplitPanelDividerLocation();
                }
            });
            editorPane.add(splitPane, BorderLayout.CENTER);
            // by default select the first item of the first list
            ((ObjectSelector) selectors[0]).selectDefaultItem();
            synchronizeEditableState();
        }
        return editorPane;
    }


    public void updateView() {
        try {
            // block all possible unwanted events during update view
            getEditorModificationSupport().setEnableEvents(false);
            if (editorPane != null) {
                titleLabel.setText(title);
            }
            for (int i = 0; i < nbrLists; i++) {
                selectors[i].updateView();
            }
            if (detailsEditor != null) detailsEditor.updateView();
        } finally {
            // allow events again
            getEditorModificationSupport().setEnableEvents(true);
        }
    }


    public void updateModel() {
        for (int i = 0; i < nbrLists; i++) {
            selectors[i].updateModel();
        }
        if (detailsEditor != null) detailsEditor.updateModel();
    }


    /**
     * Store the divider location of the spliPane into settings.
     */
    void storeSplitPanelDividerLocation() {
        Settings setts = getSettings();
        if (setts != null) {
            setts.setIntProperty(LIST_DIVIDER_LOCATION_KEY, splitPane.getDividerLocation());
        }
    }


    protected void setSelectedObject(Object selected, ObjectSelector source) {
        log.debug("Object " + selected + " selection from " + source + " (previously " + currentItem + " from " + currentSelector
                + ")");
        if ((currentItem != selected) && ((selected != null) || (currentSelector == source))) {
            if (detailsEditor != null) {
                detailsEditor.updateModel();
                currentItem = selected;
                currentSelector = source;
                detailsEditor.edit(currentItem);
            }
            // de-select on any other list
            for (int i = 0; i < nbrLists; i++) {
                if (selectors[i] != source) ((ObjectSelector) selectors[i]).clearSelection();
            }
        } else {
            log.warn(" --> selection ignored");
        }
    }


    public Object getEditedSubObject() {
        return currentItem;
    }


    @Override
    protected void synchronizeEditableState() {
        boolean newEditable = isEditable();
        boolean newEnabled = isEnabled();
        if ((editable != newEditable) || (enabled != newEnabled)) {
            editable = newEditable;
            enabled = newEnabled;
            getEditorModificationSupport().fireEditorEditabilityModified();
        }
    }


    /**
     * As there is no label in this editor, the editorComponent is the same as the objectComponent.
     */
    public JComponent getObjectComponent() {
        return getEditorComponent();
    }


    // ------------------------ EditorContext interface implementation ------------------------------------------------

    public boolean canEditInContext(boolean create, Class<?> objectClass, ObjectEditor source) {
        // The edit is done through selection in the lists and not on explicit request from contained editors
        return false;
    }


    public <T> T editInContext(T obj, Class<T> objectClass, ObjectEditor source) {
        edit(obj);
        return null;
    }


    public Object getContextObject(String objectKey) {
        return getEditorContext().getContextObject(objectKey);
    }


    public EditorFactory getEditorFactory() {
        EditorContext editorContext = getEditorContext();
        EditorFactory factory = (editorContext == null) ? null : editorContext.getEditorFactory();
        return (factory == null) ? EditorFactory.getInstance() : factory;
    }


    public Collection<ObjectEditor> getEditors() {
        return Collections.singleton((ObjectEditor) detailsEditor);
    }


    // ------------------- ObjectSelectionListener interface implementation ------------------------------------------

    public void objectSelectionChanged(ObjectSelector source) {
        setSelectedObject(source.getSelectedObject(), source);
    }

}
