/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;

import net.sf.sfac.gui.editor.EditorContext;
import net.sf.sfac.gui.framework.ActionRepository;
import net.sf.sfac.gui.framework.SharedResources;
import net.sf.sfac.gui.utils.ObjectSelectionListener;
import net.sf.sfac.gui.utils.ObjectSelector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Abstract editor for list of objects to be displayed in a table. <br>
 * Concrete implementations of this editor must provide a custom TableModel.
 * 
 * @author Olivier Berlanger
 */
public class TableObjectEditor extends SimpleObjectEditor implements ObjectSelector {

    private static final Log log = LogFactory.getLog(TableObjectEditor.class);

    private TableObjectModel tableModel;
    private List<ObjectSelectionListener> listeners;

    private JPanel objectPane;
    private JTable table;
    private JScrollPane scroll;
    private JToolBar toolbar;

    private ActionRepository repo;
    private Action viewAction;
    private Action addAction;
    private Action createAndAddAction;
    private Action removeAction;
    private Action moveUpAction;
    private Action moveDownAction;


    public TableObjectEditor() {
        setDefaultProperties();
    }


    public TableObjectEditor(TableObjectModel externalTableModel) {
        tableModel = externalTableModel;
        tableModel.setEditor(this);
        setDefaultProperties();
    }


    private void setDefaultProperties() {
        setProperty("weightx", Double.valueOf(1.0));
        setProperty("weighty", Double.valueOf(1.0));
        setProperty("fill", Integer.valueOf(GridBagConstraints.BOTH));
    }


    protected TableObjectModel createTableModel() {
        return new TableObjectModel(this);
    }


    protected TableObjectModel getTableModel() {
        if (tableModel == null) {
            tableModel = createTableModel();
        }
        return tableModel;
    }


    /**
     * Get the JTable in a scrollpane.
     */
    public JComponent getTableComponent() {
        if (scroll == null) {
            scroll = new JScrollPane(getTable());
        }
        return scroll;
    }


    /**
     * Get the JTable used by this editor.
     */
    public JTable getTable() {
        if (table == null) {
            table = createTable();
            configureTable(table);
            synchronizeEditableState();
        }
        return table;
    }


    protected JTable createTable() {
        return new JTable(getTableModel());
    }


    protected void configureTable(JTable createdTable) {
        createdTable.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    // Rem: if the table is editable, double-click fires the editor.
                    if (canViewItem() && !isEditable()) viewSelectedItem();
                }
            }
        });
        createdTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent e) {
                checkActionStates();
                fireObjectSelectionChanged();
            }

        });
        createdTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        TableColumnModel columnModel = createdTable.getColumnModel();
        int nbrColumn = columnModel.getColumnCount();
        for (int i = 0; i < nbrColumn; i++) {
            TableCellRenderer renderer = getColumnRenderer(i);
            if (renderer != null) columnModel.getColumn(i).setCellRenderer(renderer);
            TableCellEditor editor = getColumnEditor(i);
            if (editor != null) columnModel.getColumn(i).setCellEditor(editor);
        }
    }


    void checkActionStates() {
        boolean enabled = (getEditedObject() != null) && isEditable() && isEnabled();
        int selectedIndex = getSelectedIndex();
        boolean selected = (selectedIndex >= 0);
        if (viewAction != null) {
            viewAction.setEnabled(selected);
        }
        if (addAction != null) {
            addAction.setEnabled(canAddItem() && enabled);
        }
        if (createAndAddAction != null) {
            createAndAddAction.setEnabled(canCreateAndAddItem() && enabled);
        }
        if (removeAction != null) {
            removeAction.setEnabled(canRemoveItem() && enabled && selected);
        }
        if (moveUpAction != null) {
            moveUpAction.setEnabled(canSwitchItems() && enabled && selected && (selectedIndex > 0));
            moveDownAction.setEnabled(canSwitchItems() && enabled && selected
                    && (selectedIndex < getTableModel().getRowCount() - 1));
        }
    }


    protected int getSelectedIndex() {
        int selectedIndex = (table == null) ? -1 : table.getSelectedRow();
        if (selectedIndex >= getTableModel().getRowCount()) {
            selectedIndex = -1;
        }
        return selectedIndex;
    }


    public JComponent getObjectComponent() {
        if (objectPane == null) {
            objectPane = new JPanel(new GridBagLayout());
            // toolbar
            if (canViewItem() || canAddItem() || canSwitchItems()) {
                objectPane.add(getToolBar(), new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.WEST,
                        GridBagConstraints.NONE, new Insets(0, 0, 4, 0), 0, 0));
            }
            // Table
            objectPane.add(getTableComponent(), new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.WEST,
                    GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
            // update the newly created view
            synchronizeEditableState();
            updateView();
        }
        return objectPane;
    }


    protected JToolBar getToolBar() {
        if (toolbar == null) {
            repo = new ActionRepository();
            createActions(repo);
            toolbar = repo.createToolBar();
        }
        return toolbar;
    }


    @Override
    public void updateModel() {
        stopAllEditing();
    }


    void stopAllEditing() {
        if ((table != null) && table.isEditing()) table.getCellEditor().stopCellEditing();
    }


    void cancelAllEditing() {
        if ((table != null) && table.isEditing()) table.getCellEditor().stopCellEditing();
    }


    @Override
    public void updateView() {
        try {
            // block all possible unwanted events during update view
            getEditorModificationSupport().setEnableEvents(false);
            // stop any current cell edition
            stopAllEditing();
            getTableModel().fireTableDataChanged();
            checkActionStates();
        } finally {
            // allow events again
            getEditorModificationSupport().setEnableEvents(true);
        }
    }


    @SuppressWarnings("unchecked")
    protected void viewSelectedItem() {
        Object selected = getSelectedObject();
        if (selected != null) {
            getEditorContext().editInContext(selected, (Class<Object>) selected.getClass(), this);
        }
    }


    @Override
    protected void synchronizeEditableState() {
        if (table != null) {
            boolean tableEnabled = isEnabled();
            table.setEnabled(tableEnabled);
            table.setBackground(tableEnabled ? Color.WHITE : Color.LIGHT_GRAY);
            table.setForeground(tableEnabled ? Color.BLACK : Color.DARK_GRAY);
            table.repaint();
            getTableModel().setEditable(isEditable());
        }
        checkActionStates();
    }


    @SuppressWarnings("serial")
    private void createActions(ActionRepository repos) {
        // view selected Item
        if (canViewItem()) {
            viewAction = repos.addAction("viewItem", "View", "View selected item", null, null, "a0",
                    SharedResources.getIcon("view.png"), new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            viewSelectedItem();
                        }
                    });
        }
        // add new Item
        if (canAddItem()) {
            addAction = repos.addAction("addItem", "Add", "Add a new item", null, null, "a1",
                    SharedResources.getIcon("addItem.png"), new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            addItem();
                        }
                    });
        }
        // create/edit and add a new item
        if (canCreateAndAddItem()) {
            createAndAddAction = repos.addAction("createAndAddItem", "Create and add", "Create, edit and add a new item", null,
                    null, "a2", SharedResources.getIcon("newItem.png"), new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            createAndAddItem();
                        }
                    });
        }
        // remove selected Item
        if (canRemoveItem()) {
            removeAction = repos.addAction("removeItem", "Remove", "Remove selected item", null, null, "a3",
                    SharedResources.getIcon("removeItem.gif"), new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            removeItem();
                        }
                    });
        }
        // move selected Item up
        if (canSwitchItems()) {
            moveUpAction = repos.addAction("moveUp", "Move up", "Move selected item up", null, null, "a4",
                    SharedResources.getIcon("moveUp.png"), new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            moveItem(-1);
                        }
                    });
            // move selected Item down
            moveDownAction = repos.addAction("moveDown", "Move down", "Move selected item down", null, null, "a5",
                    SharedResources.getIcon("moveDown.png"), new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            moveItem(1);
                        }
                    });
        }
    }


    protected boolean canViewItem() {
        EditorContext edContext = getEditorContext();
        return (edContext != null) && edContext.canEditInContext(false, getTableModel().getItemClass(), this);
    }


    protected boolean canAddItem() {
        return getTableModel().canAddItem() && getTableModel().canSetItem();
    }


    protected boolean canCreateAndAddItem() {
        return canAddItem();
    }


    protected boolean canRemoveItem() {
        return getTableModel().canRemoveItem() && canAddItem();
    }


    protected boolean canSwitchItems() {
        return getTableModel().canSwitchItems();
    }


    protected void createAndAddItem() {
        if (getTableModel().canAddItem()) {
            int selectedIndex = getSelectedIndex();
            if ((selectedIndex < 0) || (selectedIndex > getTableModel().getRowCount())) {
                selectedIndex = getTableModel().getRowCount();
            }
            log.info("Create and add a new item at index " + selectedIndex);
            getTableModel().addItem(selectedIndex);
            Object addedItem = getEditorContext().editInContext(null, getTableModel().getItemClass(), this);
            log.debug("Selected item to add = " + addedItem);
            getTableModel().setObjectAt(selectedIndex, addedItem);
        }
    }


    protected void addItem() {
        if (canAddItem()) {
            int selectedIndex = getSelectedIndex();
            if ((selectedIndex < 0) || (selectedIndex > getTableModel().getRowCount())) {
                selectedIndex = getTableModel().getRowCount();
            }
            log.info("Add a new item at index " + selectedIndex);
            getTableModel().addItem(selectedIndex);
        }
    }


    protected void moveItem(int movment) {
        if (canSwitchItems()) {
            int selectedIndex = getSelectedIndex();
            int targetIndex = selectedIndex + movment;
            int rowCount = getTableModel().getRowCount();
            if ((selectedIndex >= 0) && (targetIndex >= 0) && (selectedIndex < rowCount) && (targetIndex < rowCount)) {
                getTableModel().switchItems(selectedIndex, targetIndex);
                table.setRowSelectionInterval(targetIndex, targetIndex);
            }
        }
    }


    protected void removeItem() {
        if (canRemoveItem()) {
            cancelAllEditing();
            int selectedIndex = getSelectedIndex();
            if ((selectedIndex >= 0) && (selectedIndex < getTableModel().getRowCount())) {
                getTableModel().removeItem(selectedIndex);
            }
        }
    }


    /**
     * Return a specific renderer for the column, or null to use the default renderer.
     */
    protected TableCellRenderer getColumnRenderer(int columnIndex) {
        return null;
    }


    /**
     * Return a specific editor for the column, or null to use the default editor.
     */
    protected TableCellEditor getColumnEditor(int columnIndex) {
        return null;
    }


    @Override
    protected Object getViewValue() {
        return null;
    }


    @Override
    protected boolean isViewReady() {
        return table != null;
    }


    @Override
    protected void setViewValue(Object newViewValue) {
    }


    // --------------------- ObjectSelector interface implementation -------------------------------------------

    public String getId() {
        return getClass().getSimpleName() + "@" + getModelAccess().getFieldPath();
    }


    public void addObjectSelectionListener(ObjectSelectionListener lis) {
        if (listeners == null) listeners = new ArrayList<ObjectSelectionListener>();
        listeners.add(lis);
    }


    public void clearSelection() {
        if (table != null) table.clearSelection();
    }


    public Object getSelectedObject() {
        int row = (table == null) ? -1 : table.getSelectedRow();
        if ((row < 0) || (row >= getTableModel().getRowCount())) return null;
        return getTableModel().getObjectAt(row);
    }


    public int getSelectedObjectCount() {
        return (table == null) ? 0 : table.getSelectedRowCount();
    }


    public List<Object> getSelectedObjects() {
        int[] row = (table == null) ? null : table.getSelectedRows();
        if (row == null) return null;
        int len = row.length;
        if (len == 0) return null;
        List<Object> selection = new ArrayList<Object>(len + 2);
        for (int i = 0; i < len; i++) {
            selection.add(getTableModel().getObjectAt(row[i]));
        }
        return selection;
    }


    public boolean isMultiSelectionEnabled() {
        return true;
    }


    public boolean canSelectObjects(List<?> toSelect) {
        Object newSelection = null;
        if ((toSelect != null) && (toSelect.size() == 1)) {
            newSelection = toSelect.get(0);
        }
        return (newSelection == null) || (getTableModel().indexOf(newSelection) >= 0);
    }


    public void selectObjects(List<?> toSelect) {
        Object newSelection = null;
        if ((toSelect != null) && (toSelect.size() == 1)) {
            newSelection = toSelect.get(0);
        }
        if (table != null) {
            if (newSelection == null) {
                table.clearSelection();
            } else {
                int selectedIndex = getTableModel().indexOf(newSelection);
                table.setRowSelectionInterval(selectedIndex, selectedIndex);
            }
        }
    }


    public void removeObjectSelectionListener(ObjectSelectionListener lis) {
        if (listeners != null) listeners.remove(lis);
    }


    public void selectDefaultItem() {
        if ((table != null) && (getTableModel().getRowCount() > 0)) {
            table.setRowSelectionInterval(0, 0);
        }
    }


    protected void fireObjectSelectionChanged() {
        if (listeners != null) {
            for (ObjectSelectionListener lis : listeners) {
                lis.objectSelectionChanged(this);
            }
        }
    }

}
