/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.editor.cmp;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;


/**
 * Abstract editor for trees. <br>
 * Concrete implementations of this editor must provide a TreeModel.
 * 
 * @author Olivier Berlanger
 */
public abstract class TreeObjectEditor extends BaseObjectEditor {

    private Object editedObject;
    private JPanel editionPane;

    private JLabel titleLabel;
    private JTree theTree;
    private TreeModel treeModel;
    private JScrollPane scroll;


    public TreeObjectEditor() {
        treeModel = createTreeModel();
    }


    protected abstract String getTitle();


    protected abstract TreeModel createTreeModel();


    protected abstract void fireTreeDataChanged();


    @Override
    public void edit(Object objToEdit) {
        editedObject = objToEdit;
        updateView();
    }


    /**
     * Get the whole editor component = a panel containig a title label and a JTree in a scrollpane.
     */
    @Override
    public Object getEditedObject() {
        return editedObject;
    }


    /**
     * Get the JTree in a scrollpane.
     */
    public JComponent getTreeComponent() {
        if (scroll == null) {
            scroll = new JScrollPane(getTree());
        }
        return scroll;
    }


    /**
     * Get the JTree used by this editor.
     */
    public JTree getTree() {
        if (theTree == null) {
            theTree = createTree();
        }
        return theTree;
    }


    protected TreeModel getTreeModel() {
        return treeModel;
    }


    protected JTree createTree() {
        JTree tree = new JTree(treeModel);
        tree.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    mouseDoubleClicked();
                }
            }
        });
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.addTreeSelectionListener(new TreeSelectionListener() {

            public void valueChanged(TreeSelectionEvent newE) {
                treeSelectionChanged();
            }

        });
        synchronizeEditableState();
        updateView();
        return tree;
    }


    public JComponent getEditorComponent() {
        buildGUI();
        return editionPane;
    }


    /**
     * Override to disable creation of a label component.
     */
    @Override
    public JLabel getLabelComponent() {
        return null;
    }


    /**
     * As there is no label in this editor, the editorComponent is the same as the objectComponent.
     */
    public JComponent getObjectComponent() {
        return getEditorComponent();
    }


    public void updateView() {
        try {
            // block all possible unwanted events during update view
            getEditorModificationSupport().setEnableEvents(false);
            if (titleLabel != null) titleLabel.setText(getTitle());
            fireTreeDataChanged();
        } finally {
            // allow events again
            getEditorModificationSupport().setEnableEvents(true);
        }
    }


    public void updateModel() {
    }


    protected void buildGUI() {
        if (editionPane == null) {
            editionPane = new JPanel(new BorderLayout());
            editionPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
            // title
            titleLabel = new JLabel();
            Font labelFont = titleLabel.getFont();
            titleLabel.setFont(labelFont.deriveFont(2f * labelFont.getSize2D()));
            editionPane.add(titleLabel, BorderLayout.NORTH);
            // Tree
            editionPane.add(getTreeComponent(), BorderLayout.CENTER);
        }
    }


    protected void mouseDoubleClicked() {
    }


    protected void treeSelectionChanged() {
        // TODO: replace by listener event
        // getEditorContext().subObjectSelected(this, getSelectedObject());
    }


    @Override
    protected void synchronizeEditableState() {
        if (theTree != null) {
            boolean treeEnabled = isEditable() && isEnabled();
            theTree.setEnabled(treeEnabled);
            theTree.setBackground(treeEnabled ? Color.WHITE : Color.LIGHT_GRAY);
            theTree.setForeground(treeEnabled ? Color.BLACK : Color.DARK_GRAY);
            theTree.repaint();
        }
    }


    public TreePath getSelectedPath() {
        if (theTree != null) return theTree.getSelectionPath();
        return null;
    }


    // ------------------- ObjectSelector interface implementation --------------------------------------

    public Object getSelectedObject() {
        Object selected = null;
        if (theTree != null) {
            TreePath selectionPath = theTree.getSelectionPath();
            selected = (selectionPath == null) ? null : selectionPath.getLastPathComponent();
        }
        return selected;
    }


    public void clearSelection() {
        if (theTree != null) {
            theTree.clearSelection();
        }
    }


    public void selectDefaultItem() {
        // if ((theTree != null) && (treeModel.getSize() > 0)) {
        // theTree.setSelectedIndex(0);
        // }
    }

}
