/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.framework;


import java.awt.Component;
import java.awt.Container;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.List;

import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPopupMenu;


/**
 * Utility class constructing a popup menu from a list of actions. This class is typically used to manage the contextual right-click
 * popup menu of a GUI component.
 * 
 * @author Olivier Berlanger
 */
public class ActionPopupMenu implements MouseListener {

    /** A dummy action used to add a separator in the popup menu. */
    public static final Action SEPARATOR = null;

    /** Array of all the actions to display in the popup menu. */
    private List<Action> actions;
    /** The (reused) popup menu */
    private JPopupMenu popup;
    /** Default parent component of the popup menu. */
    private JComponent parent;


    /**
     * Create a <code>ActionPopupMenu</code> for a specific parent component. <br>
     * In the 'allActions' action array, you can use the special <code>ActionPopupMenu.SEPARATOR</code> value to specify a separator
     * in the popup menu.
     * 
     * @param parentCmp
     *            Parent component of the popup menu.
     * @param allActions
     *            Array of all the actions to display in the popup menu.
     */
    public ActionPopupMenu(JComponent parentCmp, List<Action> allActions) {
        if (allActions == null) throw new IllegalArgumentException("Action array cannot be null");
        if (allActions.size() == 0) throw new IllegalArgumentException("Action array cannot be empty");
        parent = parentCmp;
        actions = allActions;
    }


    /**
     * Ask this <code>ActionPopupMenu</code> to register itself as mouse listener to the given component and recursively to all the
     * child components of the given component. This class will then display automatically the popup menu when it receive a
     * 'popupTrigger' event.
     * 
     * @param cmp
     *            The component to listen to.
     */
    public void addListenerOnComponents(Component cmp) {
        cmp.addMouseListener(this);
        if (cmp instanceof Container) {
            Container cnt = (Container) cmp;
            int len = cnt.getComponentCount();
            for (int i = 0; i < len; i++)
                addListenerOnComponents(cnt.getComponent(i));
        }
    }


    /**
     * Create the popup menu from the array of actions.
     * 
     * @return popup menu created from the array of actions.
     */
    private JPopupMenu createPopupMenu() {
        popup = new JPopupMenu();
        Action act;
        int len = actions.size();
        for (int i = 0; i < len; i++) {
            act = actions.get(i);
            if (act == SEPARATOR) {
                popup.addSeparator();
            } else {
                popup.add(ActionRepository.createMenuItem(act));
            }
        }
        return popup;
    }


    /**
     * Show the popup menu at the specified position in the given component coordinates.
     * 
     * @param popupParent
     *            component in which the x and y coordinates are expressed.
     * @param x
     *            X position where to show the popup menu.
     * @param y
     *            Y position where to show the popup menu.
     */
    public void showPopupMenu(Component popupParent, int x, int y) {
        if (popup == null) popup = createPopupMenu();
        popup.show(popupParent, x, y);
    }


    /**
     * Get a parent component to show the popup. Use as first choice the source of the event because the mouse event coordinate (x &
     * y) are expressed in this component coordinates.
     * 
     * @param e
     *            The mouse event.
     * @return a parent component to show the popup.
     */
    private Component getComponent(MouseEvent e) {
        Object source = e.getSource();
        if (source instanceof Component) return (Component) source;
        return parent;
    }


    // ------------------- Mouse Listener implementation ---------------------------------------------------------

    /**
     * Invoked when the mouse button has been clicked (pressed and released) on a component. If the current mouse gesture is a
     * contextual popup-menu trigger, we will show the popup menu. (The gesture depends of the current platform, ex: on windows the
     * trigger is a mouseRelease event with the right mouse button)
     * 
     * @param e
     *            the mouse event.
     */
    public void mouseClicked(MouseEvent e) {
        if (e.isPopupTrigger()) showPopupMenu(getComponent(e), e.getX(), e.getY());
    }


    /**
     * Invoked when a mouse button has been pressed on a component. If the current mouse gesture is a contextual popup-menu trigger,
     * we will show the popup menu. (The gesture depends of the current platform, ex: on windows the trigger is a mouseRelease event
     * with the right mouse button)
     * 
     * @param e
     *            the mouse event.
     */
    public void mouseReleased(MouseEvent e) {
        if (e.isPopupTrigger()) showPopupMenu(getComponent(e), e.getX(), e.getY());
    }


    /**
     * Invoked when a mouse button has been released on a component. If the current mouse gesture is a contextual popup-menu
     * trigger, we will show the popup menu. (The gesture depends of the current platform, ex: on windows the trigger is a
     * mouseRelease event with the right mouse button)
     * 
     * @param e
     *            the mouse event.
     */
    public void mousePressed(MouseEvent e) {
        if (e.isPopupTrigger()) showPopupMenu(getComponent(e), e.getX(), e.getY());
    }


    /**
     * Invoked when the mouse enters a component.
     * 
     * @param e
     *            the mouse event.
     */
    public void mouseEntered(MouseEvent e) {
    }


    /**
     * Invoked when the mouse exits a component.
     * 
     * @param e
     *            the mouse event.
     */
    public void mouseExited(MouseEvent e) {
    }

}
