/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.framework;


import java.awt.Image;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.StringTokenizer;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButton;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

import net.sf.sfac.gui.cmp.ToolbarButton;
import net.sf.sfac.gui.cmp.ToolbarToggleButton;
import net.sf.sfac.lang.LanguageListener;
import net.sf.sfac.lang.LanguageSupport;
import net.sf.sfac.string.StringUtils;
import net.sf.sfac.utils.Comparison;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Repository recording a set of actions used in the application. This repository will then be able to create the application
 * toolbar and menu bar. It will assign a mnemonic to all the actions displayed in the menu bar.
 * <p>
 * The menuItems or toolbar button order and grouping is done as following: each action is given a menuGroup and a toolbarGroup id
 * (which is a string).
 * <ul>
 * <li>An action is shown in the menu only if it has a menuGroup id, and is shown in the toolbar only if it has a toolbarGroup id.
 * <li>The order of menuItems in their menu or toolbar buttons in the toolbar follows the order of the group ids.
 * <li>The menuItems or toolbar button are grouped by first letter of their group id. (a separator is added between the groups).
 * </ul>
 * For actions corresponding to menuItems you can specify all the menu hierarchy where the menuItem has to be - the actions for the
 * corresponding menus will be created automatically. The menus names are separated by a slash '/'. For each menu in the menu
 * hierarchy you have to specify also the menuGroup. So the given menuGroup is in fact a string containing a slash-separated list of
 * menuGroup ids.
 * </p>
 * <p>
 * If a LanguageSupport is configured, this class will listen for language changes and update the actions accordingly (which will
 * update the menus, menuItems and toolbar buttons).
 * </p>
 * <p>
 * Some classes can register as BarChangeListener, they will then be notified when the menuBar or the Toolbar is created. So they
 * have the opportunity to add custom components to the application bars.
 * </p>
 * 
 * @author Olivier Berlanger
 */
public class ActionRepository implements LanguageListener {

    private static Log log = LogFactory.getLog(ActionRepository.class);

    /** Special keys for custom actions values: ID of the action. */
    public static final String ACTION_ID = "ActionID";
    /** Special keys for custom actions values: menu group id. */
    public static final String MENU_GROUP = "MenuActionGroup";
    /** Special keys for custom actions values: toolbar group id. */
    public static final String TOOLBAR_GROUP = "ToolbarActionGroup";
    /** Special keys for custom actions values: button model (for toggle or radio buttons). */
    public static final String BUTTON_MODEL = "ButtonModel";
    /** Special keys for custom actions values: button model (for radio buttons). */
    public static final String BUTTON_GROUP = "ButtonGroup";
    /** Special keys for custom actions values: List holding sub-actions (for menus). */
    public static final String SUB_ACTIONS = "subMenuActions";

    /** Map holding all the actions of this repository with the action ID as key. */
    private Map<String, Action> allActions;
    /** All the actions defining a toolbar group id. */
    private List<Action> toolbarActions;
    /** All the actions defining a menu group id. */
    private List<Action> menuActions;
    /** The created menu bar. */
    private JMenuBar menuBar;
    /** All listeners for 'bar change' events. */
    private List<BarChangeListener> listeners;
    /** The mnemonic manager */
    private MnemonicManager mnemonicMgr;
    /** The base level used in mnemonic manager for calculating menu bar mnemonics. */
    private int baseMnemonicLevel;


    /**
     * Create an empty action repository (not multilingual).
     */
    public ActionRepository() {
        allActions = new HashMap<String, Action>();
        toolbarActions = new ArrayList<Action>();
        menuActions = new ArrayList<Action>();
        LanguageSupport.addLanguageListener(this);
    }


    public void setMnemonicManager(MnemonicManager manager, int baseLevel) {
        mnemonicMgr = manager;
        baseMnemonicLevel = baseLevel;
    }


    public MnemonicManager getMnemonicManager() {
        if (mnemonicMgr == null) mnemonicMgr = new MnemonicManager();
        return mnemonicMgr;
    }


    public int getBaseMnemonicLevel() {
        return baseMnemonicLevel;
    }


    public int getActionCount() {
        return allActions.size();
    }


    public int getToolbarActionCount() {
        return toolbarActions.size();
    }


    public List<Action> getToolbarActions() {
        return toolbarActions;
    }


    public int getMenuActionCount() {
        return menuActions.size();
    }


    public List<Action> getMenuActions() {
        return menuActions;
    }


    /**
     * Get an action of this repository.
     * 
     * @param actionId
     *            id of the action.
     * @return the action of this repository with the given id.
     */
    public Action getAction(String actionId) {
        return allActions.get(actionId);
    }


    /**
     * Create the application menu bar.
     * 
     * @return the application menu bar.
     */
    public JMenuBar createMenuBar() {
        menuBar = new JMenuBar();
        sortMenuActions(menuActions, new ActionComparator(MENU_GROUP));
        setAllMnemonics();
        addMenusToMenuBar();
        fireMenuBarCreated(menuBar);
        return menuBar;
    }


    /**
     * Create the application toolbar.
     * 
     * @return the application toolbar.
     */
    public JToolBar createToolBar() {
        JToolBar toolBar = new JToolBar("mainToolBar");
        Collections.sort(toolbarActions, new ActionComparator(TOOLBAR_GROUP));
        int len = toolbarActions.size();
        int currentActionGroup = -1;
        int actionGroup;
        for (int i = 0; i < len; i++) {
            Action act = toolbarActions.get(i);
            actionGroup = getGroup(act, TOOLBAR_GROUP);
            if (currentActionGroup != actionGroup) {
                if (i > 0) toolBar.addSeparator();
                currentActionGroup = actionGroup;
            }
            AbstractButton but = createToolbarButton(act);
            toolBar.add(but);
        }
        toolBar.setFloatable(false);
        fireToolBarCreated(toolBar);
        return toolBar;
    }


    @Override
    public String toString() {
        return "ActionRepository@" + Integer.toHexString(System.identityHashCode(this));
    }


    // ---------------------------- Static helpers --------------------------------------------------------

    /**
     * Set a toggle action selected or unselected. A toggle action is an action that has been configured with either
     * <code>setToggleAction(Action act)</code> or <code>setRadioAction(Action act, ButtonGroup group)</code>. This will set all
     * buttons or menuItems attached to the action selected or unselected as well.
     * 
     * @param act
     *            the toggle action.
     * @param selected
     *            true if the action has to be selected, false otherwise.
     */
    public static void setActionSelected(Action act, boolean selected) {
        ButtonModel butModel = (ButtonModel) act.getValue(ActionRepository.BUTTON_MODEL);
        if (butModel == null) throw new IllegalArgumentException("Action '" + act.getValue(Action.NAME) + "' cannot be selected");
        if (butModel.isSelected() != selected) butModel.setSelected(selected);
    }


    /**
     * Check if a toggle action is selected or unselected. A toggle action is an action that has been configured with either
     * <code>setToggleAction(Action act)</code> or <code>setRadioAction(Action act, ButtonGroup group)</code>.
     * 
     * @param act
     *            the action to check
     * @return true if the action is selected, false otherwise.
     */
    public static boolean isActionSelected(Action act) {
        ButtonModel butModel = (ButtonModel) act.getValue(ActionRepository.BUTTON_MODEL);
        if (butModel == null) throw new IllegalArgumentException("Action '" + act.getValue(Action.NAME) + "' cannot be selected");
        return butModel.isSelected();
    }


    /**
     * Set an action as Toggle Action. Toggle actions will be shown as JToggleButton in the toolbar and JCheckBoxMenuItem in the
     * menu bar. When an action is a toggle action, you can check it's state with <code>isActionSelected(Action act)</code> and set
     * it's state with <code>setActionSelected(Action act, boolean selected)</code>.
     * 
     * @param act
     *            the action to configure as toggle action.
     */
    public static void setToggleAction(Action act) {
        act.putValue(BUTTON_MODEL, new JToggleButton.ToggleButtonModel());
    }


    /**
     * Set an action as radio Action. Radio actions will be shown as JToggleButton in the toolbar and JRadioButtonMenuItem in the
     * menu bar and they will be correctly added in the given button group. When an action is a radio action, you can check it's
     * state with <code>isActionSelected(Action act)</code> and set it's state with
     * <code>setActionSelected(Action act, boolean selected)</code>.
     * 
     * @param act
     *            the action to configure as toggle action.
     * @param group
     *            Button Group of the radio action.
     */
    public static void setRadioAction(Action act, ButtonGroup group) {
        act.putValue(BUTTON_MODEL, new JToggleButton.ToggleButtonModel());
        act.putValue(BUTTON_GROUP, group);
    }


    /**
     * Resize an ImageIcon to the given size
     * 
     * @param icon
     *            the ImageIcon to resized.
     * @param newSize
     *            the new size (width==height)
     * @return an new ImageIcon with the required size
     */
    public static ImageIcon resizeIcon(ImageIcon icon, int newSize) {
        Image im = icon.getImage();
        return new ImageIcon(im.getScaledInstance(newSize, newSize, Image.SCALE_REPLICATE));
    }


    public static void registerKeyStrokes(Action act, JComponent cmp) {
        Object ks = act.getValue(Action.ACCELERATOR_KEY);
        if (ks != null) {
            InputMap windowMap = cmp.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
            ActionMap actionMap = cmp.getActionMap();
            String key = (String) act.getValue(Action.ACTION_COMMAND_KEY);
            if (Comparison.isEmpty(key)) {
                key = (String) act.getValue(Action.NAME);
                if (Comparison.isEmpty(key)) {
                    throw new IllegalStateException("No command or name for action: " + act);
                }
                act.putValue(Action.ACTION_COMMAND_KEY, key);
            }
            if (ks instanceof KeyStroke) {
                KeyStroke stroke = (KeyStroke) ks;
                windowMap.put(stroke, key);
            } else if (ks instanceof KeyStroke[]) {
                KeyStroke[] strokes = (KeyStroke[]) ks;
                for (KeyStroke stroke : strokes) {
                    windowMap.put(stroke, key);
                }
            } else {
                throw new IllegalStateException("Bad value for attribute: " + Action.ACCELERATOR_KEY + " in action " + key + " = "
                        + ks);
            }
            actionMap.put(key, act);
        }
    }


    // ------------------------ Action creation ----------------------------------------------------------------------

    /**
     * Configure a multi-lingual action (i.e. add all the needed Action values) and add it in the repository. <br>
     * The language manager must be defined and contain the following entries:
     * <ul>
     * <li>actionId+"_NAME" : for the action name.
     * <li>actionId+"_DESCR" : for the action description (optional).
     * <li>menuId+"_MENU" : name of the menu (for all the menu ids appearing in menuPath).
     * </ul>
     * 
     * @param actionId
     *            ID of the action.
     * @param menuPath
     *            slash-sperated list of menu ids of the menus holding menuItem of this action. an action will be created for each
     *            specified menu. (null if action doesn't have to appear in a menu)
     * @param menuGroup
     *            slash-sperated list of menuGroup ids of the menus holding menuItem of this action plus menuGroup id for this
     *            action. (null if action doesn't have to appear in a menu)
     * @param toolbarGroup
     *            toolbar group id of this action. (null if action doesn't have to appear in toolbar)
     * @param actionIcon
     *            Icon of the action.
     * @param act
     *            the action instance (not configured).
     * @return the action configured action instance.
     */
    public Action addAction(String actionId, String menuPath, String menuGroup, String toolbarGroup, Icon actionIcon, Action act) {
        return addAction(actionId, null, null, menuPath, menuGroup, toolbarGroup, actionIcon, act);
    }


    /**
     * Configure an action (i.e. add all the needed Action values) and add it in the repository. <br>
     * 
     * @param actionId
     *            ID of the action.
     * @param actionName
     *            Displayed name of the action.
     * @param actionDescr
     *            Description of the action (for menuItem tooltip).
     * @param menuPath
     *            slash-sperated list of menu ids/names of the menus holding menuItem of this action. an action will be created for
     *            each specified menu. (null if action doesn't have to appear in a menu)
     * @param menuGroup
     *            slash-sperated list of menuGroup ids of the menus holding menuItem of this action plus menuGroup id for this
     *            action. (null if action doesn't have to appear in a menu)
     * @param toolbarGroup
     *            toolbar group id of this action. (null if action doesn't have to appear in toolbar)
     * @param actionIcon
     *            Icon of the action.
     * @param act
     *            the action instance (not configured).
     * @return the action configured action instance.
     */
    @SuppressWarnings("unchecked")
    public Action addAction(String actionId, String actionName, String actionDescr, String menuPath, String menuGroup,
            String toolbarGroup, Icon actionIcon, Action act) {
        act.putValue(ACTION_ID, actionId);
        setActionNameAndDescription(actionId, act, actionName, actionDescr);
        allActions.put(actionId, act);
        if (actionIcon != null) act.putValue(Action.SMALL_ICON, actionIcon);
        // menu configuration
        if (menuPath != null) {
            // create same structure for actions as for the real menu.
            // each JMenu is represented by a action containing arrayList of sub-actions in the SUB_ACTIONS value.
            String[] menuIds = tokenize(menuPath);
            String[] menuGroups = tokenize(menuGroup);
            int menuLen = menuIds.length;
            Action menuAction = getMenuAction(menuIds, menuGroups, menuLen - 1);
            List<Action> parentArray = (List<Action>) menuAction.getValue(SUB_ACTIONS);
            String actionGroup = (menuLen < menuGroups.length) ? menuGroups[menuLen] : "z8";
            act.putValue(MENU_GROUP, actionGroup);
            parentArray.add(act);
        }
        // toolbar configuration
        if (toolbarGroup != null) {
            act.putValue(TOOLBAR_GROUP, toolbarGroup);
            toolbarActions.add(act);
        }
        return act;
    }


    /**
     * Get (and recursively create if necessary) the menu action associated to the given menu path.
     * 
     * @param menuPath
     *            slash-sperated list of menu ids/names of the menus holding menuItem of this action. an action will be created for
     *            each specified menu. (null if action doesn't have to appear in a menu)
     * @param menuGroup
     *            slash-sperated list of menuGroup ids of the menus holding menuItem of this action plus menuGroup id for this
     *            action. (null if action doesn't have to appear in a menu)
     * @return the requested menu action (the last in the path).
     */
    public Action getMenuAction(String menuPath, String menuGroup) {
        String[] menuIds = tokenize(menuPath);
        String[] menuGroups = tokenize(menuGroup);
        int menuLen = menuIds.length;
        Action menuAction = getMenuAction(menuIds, menuGroups, menuLen - 1);
        return menuAction;
    }


    // //////// private Toolbar/Menubar creation ////////////////////////////////////////////////////////////////////

    /**
     * Create all JMenus and them to menu bar.
     */
    @SuppressWarnings("unchecked")
    private void addMenusToMenuBar() {
        JMenu menu;
        Action act;
        List<Action> subMenuActions;
        int len = menuActions.size();
        // recursively create the menus
        for (int i = 0; i < len; i++) {
            act = menuActions.get(i);
            menu = new JMenu(act);
            subMenuActions = (List<Action>) act.getValue(SUB_ACTIONS);
            addMenuItems(menu, subMenuActions);
            menuBar.add(menu);
        }
    }


    /**
     * Add all menu items to a menu.
     * 
     * @param menu
     *            the parent menu
     * @param menuItemActions
     *            list of all actions belonging to the given menu.
     */
    @SuppressWarnings("unchecked")
    private void addMenuItems(JMenu menu, List<Action> menuItemActions) {
        Action act;
        List<Action> subMenuActions;
        JMenuItem menuItem;
        JMenu subMenu;
        String actionMenuName;
        int group;
        int currentGroup = -1;

        int len = menuItemActions.size();
        // first create menus to assign them mainMnemonics first
        for (int i = 0; i < len; i++) {
            act = menuItemActions.get(i);
            actionMenuName = (String) act.getValue(Action.NAME);
            group = getGroup(act, MENU_GROUP);
            if (currentGroup != group) {
                if (i > 0) menu.addSeparator();
                currentGroup = group;
            }
            subMenuActions = (List<Action>) act.getValue(SUB_ACTIONS);
            if (subMenuActions == null) {
                // menu item
                if (log.isDebugEnabled()) log.debug("Create menu item '" + actionMenuName + "', group='" + act.getValue(MENU_GROUP)
                        + "'");
                menuItem = createMenuItem(act);
                menu.add(menuItem);
            } else {
                // sub menu
                if (log.isDebugEnabled()) log.debug("Create menu '" + actionMenuName + "', group='" + act.getValue(MENU_GROUP)
                        + "'");
                subMenu = new JMenu(act);
                addMenuItems(subMenu, subMenuActions);
                menu.add(subMenu);
            }
        }
    }


    /**
     * Set mnemonics for all actions of the menus.
     */
    private void setAllMnemonics() {
        if (mnemonicMgr == null) {
            mnemonicMgr = new MnemonicManager();
            baseMnemonicLevel = 0;
        } else {
            mnemonicMgr.clear(baseMnemonicLevel);
        }
        fireBeforeMnemonicAssignment();
        setMenuMnemonics(menuActions, baseMnemonicLevel);
    }


    /**
     * Set mnemonics for a given menu.
     * 
     * @param menuItemActions
     *            list of all actions belonging to the given menu.
     * @param level
     *            current level in the stack.
     */
    @SuppressWarnings("unchecked")
    private void setMenuMnemonics(List<Action> menuItemActions, int level) {
        Action act;
        List<Action> subMenuActions;
        int len = menuItemActions.size();
        // first assign mnemonics to this level.
        for (int i = 0; i < len; i++) {
            act = menuItemActions.get(i);
            mnemonicMgr.assignActionMnemonic(act, level);
        }
        // then check for sub-levels
        int nextLevel = level + 1;
        for (int i = 0; i < len; i++) {
            act = menuItemActions.get(i);
            subMenuActions = (List<Action>) act.getValue(SUB_ACTIONS);
            if (subMenuActions != null) {
                mnemonicMgr.clear(nextLevel);
                setMenuMnemonics(subMenuActions, nextLevel);
            }
        }
    }


    /**
     * Get the group of an action (the first char of the group id).
     * 
     * @param act
     *            the action.
     * @param groupKey
     *            key for the group in the action (MENU_GROUP or TOOLBAR_GROUP).
     * @return the group of the given action
     */
    public static int getGroup(Action act, String groupKey) {
        String group = (String) act.getValue(groupKey);
        if ((group == null) || (group.length() == 0)) return -1;
        return group.charAt(0);
    }


    /**
     * Recursively sort the menu actions of a list.
     * 
     * @param list
     *            the list holding menu actions.
     * @param comparator
     *            the menu actions comparator.
     */
    @SuppressWarnings("unchecked")
    private void sortMenuActions(List<Action> list, ActionComparator comparator) {
        Collections.sort(list, comparator);
        int len = list.size();
        Action act;
        List<Action> subList;
        for (int i = 0; i < len; i++) {
            act = list.get(i);
            subList = (List<Action>) act.getValue(SUB_ACTIONS);
            if (subList != null) sortMenuActions(subList, comparator);
        }
    }


    /**
     * Get the name of a menu. As we don't know if the given id is the name of the menu or the id of the menu, we just have to try
     * to translate the given id and use it as the name if no translation is found.
     * 
     * @param actionMenuId
     *            name or id of the menu.
     * @return the name of the menu.
     */
    private String getMenuName(String actionMenuId) {
        String menuNameKey = actionMenuId + "_MENU";
        return LanguageSupport.getOptionalLocalizedString(menuNameKey, actionMenuId);
    }


    /**
     * Create a standard button associated to the given action. The created button will be JButton, JToggleButton or JRadioButton
     * depending of the action type.
     * 
     * @param act
     *            the action.
     * @return a toolbar button associated to the given action.
     */
    public static AbstractButton createButton(Action act) {
        AbstractButton but;
        ButtonModel butModel = (ButtonModel) act.getValue(BUTTON_MODEL);
        if (butModel == null) {
            if (log.isDebugEnabled()) log.debug("Create button for action '" + act.getValue(Action.NAME) + "', group='"
                    + act.getValue(TOOLBAR_GROUP) + "'");
            but = new JButton(act);
        } else {
            ButtonGroup butGroup = (ButtonGroup) act.getValue(BUTTON_GROUP);
            if (butGroup == null) {
                if (log.isDebugEnabled()) log.debug("Create toggle button for action '" + act.getValue(Action.NAME) + "', group='"
                        + act.getValue(TOOLBAR_GROUP) + "'");
                but = new JToggleButton(act);
                but.setModel(butModel);
            } else {
                if (log.isDebugEnabled()) log.debug("Create radio button for action '" + act.getValue(Action.NAME) + "', group='"
                        + act.getValue(TOOLBAR_GROUP) + "'");
                but = new JRadioButton(act);
                but.setModel(butModel);
                butGroup.add(but);
            }
        }
        return but;
    }


    /**
     * Create a toolbar button associated to the given action. The created button will be ToolbarButton or ToolbarToggleButton
     * depending of the action type.
     * 
     * @param act
     *            the action.
     * @return a toolbar button associated to the given action.
     */
    public static AbstractButton createToolbarButton(Action act) {
        AbstractButton but;
        ButtonModel butModel = (ButtonModel) act.getValue(BUTTON_MODEL);
        if (butModel == null) {
            if (log.isDebugEnabled()) log.debug("Create toolbar button for action '" + act.getValue(Action.NAME) + "', group='"
                    + act.getValue(TOOLBAR_GROUP) + "'");
            but = new ToolbarButton(act);
        } else {
            if (log.isDebugEnabled()) log.debug("Create toolbar toggle button for action '" + act.getValue(Action.NAME)
                    + "', group='" + act.getValue(TOOLBAR_GROUP) + "'");
            but = new ToolbarToggleButton(act, butModel);
            ButtonGroup butGroup = (ButtonGroup) act.getValue(BUTTON_GROUP);
            if (butGroup != null) butGroup.add(but);
        }
        return but;
    }


    /**
     * Create a menuItem associated to the given action. The created menuItem will be JMenuItem, JCheckBoxMenuItem or
     * JRadioButtonMenuItem depending of the action type.
     * 
     * @param act
     *            the action.
     * @return a menuItem associated to the given action.
     */
    public static JMenuItem createMenuItem(Action act) {
        JMenuItem item;
        ButtonModel butModel = (ButtonModel) act.getValue(BUTTON_MODEL);
        if (butModel == null) {
            item = new JMenuItem();
        } else {
            ButtonGroup butGroup = (ButtonGroup) act.getValue(BUTTON_GROUP);
            if (butGroup == null) {
                item = new JCheckBoxMenuItem();
                item.setModel(butModel);
            } else {
                item = new JRadioButtonMenuItem();
                item.setModel(butModel);
                butGroup.add(item);
            }
        }
        item.setAction(act);
        // check the icon size (max=16 pixels)
        Icon ico = (Icon) act.getValue(Action.SMALL_ICON);
        if ((ico != null) && ((ico.getIconHeight() > 16) || (ico.getIconWidth() > 16)) && (ico instanceof ImageIcon)) {
            ImageIcon imIco = (ImageIcon) ico;
            item.setIcon(resizeIcon(imIco, 16));
        }
        return item;
    }


    /**
     * Tokenize a slash-separated string. The returned array is guaranteed to be non-null but can be zero-sized.
     * 
     * @param source
     *            source string.
     * @return string tokenized to a string array.
     */
    private String[] tokenize(String source) {
        String[] ids;
        if (source == null) return new String[0];
        if (source.indexOf('/') >= 0) {
            StringTokenizer tokenizer = new StringTokenizer(source, "/");
            int len = tokenizer.countTokens();
            ids = new String[len];
            for (int i = 0; i < len; i++)
                ids[i] = tokenizer.nextToken();
        } else {
            ids = new String[] { source };
        }
        return ids;
    }


    /**
     * Get (and recursively create if necessary) the menu action associated to the given menu path.
     * 
     * @param menuIds
     *            array with ids/names of the menus hierarchy.
     * @param menuGroups
     *            corresponding menu group ids for the given menus.
     * @param index
     *            max index to go in the array (used for recursion)
     * @return the menu action with the id at 'index' position in 'menuIds'.
     */
    @SuppressWarnings({ "unchecked", "serial" })
    private Action getMenuAction(String[] menuIds, String[] menuGroups, int index) {
        Action subMenuAct = null;
        String menuId = menuIds[index];
        List<Action> parentArray;
        if (index == 0) parentArray = menuActions;
        else {
            Action act = getMenuAction(menuIds, menuGroups, index - 1);
            parentArray = (List<Action>) act.getValue(SUB_ACTIONS);
        }
        // search if already exists
        int len = parentArray.size();
        Action act;
        for (int i = 0; i < len; i++) {
            act = parentArray.get(i);
            if (menuId.equals(act.getValue(ACTION_ID))) {
                subMenuAct = act;
                break;
            }
        }
        // create if not found
        if (subMenuAct == null) {
            subMenuAct = new AbstractAction() {

                public void actionPerformed(ActionEvent ae) {
                }
            };
            String menuGroup = (index < menuGroups.length) ? menuGroups[index] : "z9";
            String menuName = getMenuName(menuId);
            subMenuAct.putValue(ACTION_ID, menuId);
            subMenuAct.putValue(Action.NAME, menuName);
            subMenuAct.putValue(Action.SHORT_DESCRIPTION, menuName);
            subMenuAct.putValue(MENU_GROUP, menuGroup);
            subMenuAct.putValue(SUB_ACTIONS, new ArrayList<Action>());
            parentArray.add(subMenuAct);
            allActions.put(menuId, subMenuAct);
        }
        return subMenuAct;
    }


    // ----------------- LanguageListener interface implementation --------------------------------------------------------

    /**
     * Notification that the current application language has changed. Update all the language-dependent text in the actions. This
     * will trigger updates to associated UI components.
     * 
     * @param mgr
     *            the language manager triggering the notification (should be the same as the 'languageMgr' instance variable)
     * @param newLocale
     *            the new locale.
     */
    public void languageChanged(Locale newLocale) {
        // update text of all actions
        for (Iterator<String> it = allActions.keySet().iterator(); it.hasNext();) {
            String actionId = it.next();
            Action act = allActions.get(actionId);
            if (act.getValue(SUB_ACTIONS) == null) {
                // final action
                setActionNameAndDescription(actionId, act, null, null);
            } else {
                // menu action
                String nameKey = actionId + "_MENU";
                String name = LanguageSupport.getLocalizedString(nameKey);
                if (name != nameKey) {
                    act.putValue(Action.NAME, name);
                    act.putValue(Action.SHORT_DESCRIPTION, name);
                }
            }
        }
        setAllMnemonics();
        // repaint menu bar
        if (menuBar != null) {
            menuBar.revalidate();
            menuBar.repaint();
        }
    }


    private void setActionNameAndDescription(String actionId, Action act, String directName, String directDescription) {
        // set name
        String nameKey = actionId + "_NAME";
        String name = LanguageSupport.getLocalizedString(nameKey);
        if (name == nameKey) {
            name = Comparison.isDefined(directName) ? directName : StringUtils.firstToUpperCase(actionId);
        }
        act.putValue(Action.NAME, name);
        // set description
        String descrKey = actionId + "_DESCR";
        String descr = LanguageSupport.getLocalizedString(descrKey);
        if (descr == descrKey) {
            descr = Comparison.isDefined(directDescription) ? directDescription : name;
        }
        act.putValue(Action.SHORT_DESCRIPTION, descr);
        act.putValue(Action.LONG_DESCRIPTION, descr);
    }

    // -------------------- Comparator class used to sort actions ----------------------------------------

    /**
     * Comparator for actions using the action menuGroup id or toolbarGroup id.
     * 
     * @author Olivier Berlanger
     */
    static class ActionComparator implements Comparator<Action> {

        /** Key to the compared id in the action (MENU_GROUP or TOOLBAR_GROUP). */
        private String compareKey;


        /**
         * Create a comparator for the given group.
         * 
         * @param key
         *            Key to the compared id in the action (MENU_GROUP or TOOLBAR_GROUP).
         */
        ActionComparator(String key) {
            compareKey = key;
        }


        /**
         * Compare to actions by comparing their group ids.
         * 
         * @param obj1
         *            action 1.
         * @param obj2
         *            action 2
         * @return 0, 1 or -1.
         */
        public int compare(Action act1, Action act2) {
            String val1 = (String) act1.getValue(compareKey);
            String val2 = (String) act2.getValue(compareKey);
            return val1.compareTo(val2);
        }

    }


    // -------------------- BarChange listener provider implementation ----------------------------------------

    /**
     * Add a BarChangeListener which will be notified when either toolbar or menubar will be created.
     * 
     * @param lis
     *            a BarChangeListener.
     */
    public void addBarChangeListener(BarChangeListener lis) {
        if (listeners == null) listeners = new ArrayList<BarChangeListener>();
        listeners.add(lis);
    }


    /**
     * Remove a previously added BarChangeListener. (does nothing if the gievn BarChangeListener was not added).
     * 
     * @param lis
     *            a BarChangeListener.
     */
    public void removeBarChangeListener(BarChangeListener lis) {
        if (listeners != null) listeners.remove(lis);
    }


    /**
     * Fire notification that the mnemonics will be updated to all registered BarChangeListener.
     */
    private void fireBeforeMnemonicAssignment() {
        int len = (listeners == null) ? 0 : listeners.size();
        if (log.isDebugEnabled()) log.debug("Fire menuBar created to " + len + " listeners");
        for (int i = 0; i < len; i++) {
            BarChangeListener lis = listeners.get(i);
            lis.beforeMnemonicAssignment(this);
        }
    }


    /**
     * Fire notification that the menuBar has changed to all registered BarChangeListener.
     * 
     * @param menubar
     *            the created/updated menuBar.
     */
    private void fireMenuBarCreated(JMenuBar menubar) {
        int len = (listeners == null) ? 0 : listeners.size();
        if (log.isDebugEnabled()) log.debug("Fire menuBar created to " + len + " listeners");
        for (int i = 0; i < len; i++) {
            BarChangeListener lis = listeners.get(i);
            lis.menubarCreated(this, menubar);
        }
    }


    /**
     * Fire notification that the toolbar has changed to all registered BarChangeListener.
     * 
     * @param toolbar
     *            the created/updated toolbar.
     */
    private void fireToolBarCreated(JToolBar toolbar) {
        int len = (listeners == null) ? 0 : listeners.size();
        if (log.isDebugEnabled()) log.debug("Fire toolBar created to " + len + " listeners");
        for (int i = 0; i < len; i++) {
            BarChangeListener lis = listeners.get(i);
            lis.toolbarCreated(this, toolbar);
        }
    }

}
