/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.framework;


import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Locale;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JToolBar;

import net.sf.sfac.lang.LanguageListener;
import net.sf.sfac.lang.LanguageSupport;
import net.sf.sfac.setting.Settings;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * (Main) frame of an application. <br>
 * This base class manage following settings:
 * <ul>
 * <li>frame bounds.
 * <li>frame maximized.
 * <li>toolbar visible.
 * </ul>
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class ApplicationFrame extends JFrame implements LanguageListener {

    private static Log log = LogFactory.getLog(ApplicationFrame.class);

    /** Key used to store frame bounds in settings. */
    private static final String FRAME_BOUNDS_KEY = ".bounds";
    /** Key used to store frame bounds in settings. */
    private static final String FRAME_MAXIMIZED_KEY = ".maximized";
    /** Key used to store toolbar state (visible/not visible) in settings. */
    private static final String VIEW_TOOLBAR_KEY = ".view.toolBar";
    /** Default bounds of the frame */
    private static final Rectangle DEFAULT_BOUNDS = new Rectangle(20, 20, 850, 650);

    /** Current toolbar of the frame. */
    private JToolBar toolBar;
    /** Current settings holder (can be null). */
    private Settings setts;
    /** Action for managing toolbar state (visible/not visible). */
    private Action viewToolbarAction;
    /** current toolbar state (visible/not visible). */
    private boolean toolbarVisible;


    /**
     * Construct an application frame. If the given settings holder is not null, this frame will store 2 settings:
     * <ul>
     * <li>The frame bounds.
     * <li>The toolbar state (visible/not visible)
     * </ul>
     * The frame title is fetch from labguage support using the "" key.
     * 
     * @param sett
     *            settings holder object (can be null if this frame does't have to store settings).
     */
    public ApplicationFrame(Settings sett) {
        this(LanguageSupport.getLocalizedString("FRAME_TITLE"), sett);
        LanguageSupport.addLanguageListener(this);
    }


    /**
     * Construct an application frame. If the given settings holder is not null, this frame will store 2 settings:
     * <ul>
     * <li>The frame bounds.
     * <li>The toolbar state (visible/not visible)
     * </ul>
     * 
     * @param title
     *            Frame title.
     * @param sett
     *            settings holder object (can be null if this frame does't have to store settings).
     */
    public ApplicationFrame(String title, Settings sett) {
        super();
        setts = sett;
        setTitle(title);
        toolbarVisible = true;
        if (setts != null) setFrameBounds();
        else setBounds(DEFAULT_BOUNDS);
    }


    public Settings getSettings() {
        return setts;
    }


    /**
     * Prefix for keys used to store settings. <br>
     * Subclasses can override this method to have their own settings.
     * 
     * @return Prefix for keys used to store settings.
     */
    protected String getKeyPrefix() {
        return "frame";
    }


    @Override
    public String toString() {
        return getClass().getSimpleName() + "[" + getKeyPrefix() + "]";
    }


    /**
     * Set bounds of this frame and add component listnener to be notified of bound changes (so we can store last bounds in
     * settings). This method is called only if the settings holder is not null.
     */
    private void setFrameBounds() {
        if (log.isDebugEnabled()) log.debug("Set frame bounds to: "
                + setts.getRectangleProperty(getKeyPrefix() + FRAME_BOUNDS_KEY, DEFAULT_BOUNDS));
        setBounds(setts.getRectangleProperty(getKeyPrefix() + FRAME_BOUNDS_KEY, DEFAULT_BOUNDS));
        if (setts.getBooleanProperty(getKeyPrefix() + FRAME_MAXIMIZED_KEY, false)) setExtendedState(Frame.MAXIMIZED_BOTH);
        addComponentListener(new ComponentAdapter() {

            @Override
            public void componentResized(ComponentEvent e) {
                storeFrameBounds();
            }


            @Override
            public void componentMoved(ComponentEvent e) {
                storeFrameBounds();
            }
        });
    }


    void storeFrameBounds() {
        boolean maximized = getExtendedState() == Frame.MAXIMIZED_BOTH;
        if (!maximized) setts.setRectangleProperty(getKeyPrefix() + FRAME_BOUNDS_KEY, getBounds());
        setts.setBooleanProperty(getKeyPrefix() + FRAME_MAXIMIZED_KEY, maximized);
    }


    /**
     * Set the toolbar of the frame (it will be added at the north position of the border layout).
     * 
     * @param toolb
     *            the toolbar of the frame.
     */
    public void setToolBar(JToolBar toolb) {
        toolBar = toolb;
        toolBar.setVisible(toolbarVisible);
        getContentPane().add(toolBar, BorderLayout.NORTH);
    }


    /**
     * Set the the toolbar/non-visible.
     * 
     * @param visible
     *            iff the toolbar should be visible.
     */
    public void setToolBarVisible(boolean visible) {
        if (toolbarVisible != visible) {
            if (log.isDebugEnabled()) log.debug("Set toolbar visible = " + visible);
            if (toolBar != null) {
                toolBar.setVisible(visible);
                JPanel contentPane = (JPanel) getContentPane();
                contentPane.revalidate();
                contentPane.repaint();
            }
            toolbarVisible = visible;
            if (setts != null) setts.setBooleanProperty(getKeyPrefix() + VIEW_TOOLBAR_KEY, toolbarVisible);
            if (viewToolbarAction != null) ActionRepository.setActionSelected(viewToolbarAction, toolbarVisible);
        }
    }


    /**
     * Create and add to the given repository an action to manage toolbar state (visible/not visible) to the given repository.
     * 
     * @param repo
     *            the repository for all application actions.
     * @param menuPath
     *            path to the menu where a menuItem should be added.
     * @param menuGroup
     *            group ids of the menu and menuItems.
     * @param toolBarGroup
     *            group id of the toolbar action.
     * @see ActionRepository#addAction(String, String, String, String, Icon, Action)
     */
    public void createViewToolbarAction(ActionRepository repo, String menuPath, String menuGroup, String toolBarGroup) {
        viewToolbarAction = new AbstractAction() {

            public void actionPerformed(ActionEvent ae) {
                boolean visible = ActionRepository.isActionSelected(this);
                setToolBarVisible(visible);
            }
        };
        Icon ico = SharedResources.getIcon("viewToolBar.png");
        if (LanguageSupport.getOptionalLocalizedString("viewToolBar_NAME", null) == null) {
            repo.addAction("viewToolBar", "Actions Toolbar", "Show/hide actions toolbar.", menuPath, menuGroup, toolBarGroup, ico,
                    viewToolbarAction);
        } else {
            repo.addAction("viewToolBar", menuPath, menuGroup, toolBarGroup, ico, viewToolbarAction);
        }
        ActionRepository.setToggleAction(viewToolbarAction);
        ActionRepository.setActionSelected(viewToolbarAction, toolbarVisible);
    }


    // -------------------- LanguageListener interface implementation -----------------------------------------------------

    public void languageChanged(Locale newLocale) {
        setTitle(LanguageSupport.getOptionalLocalizedString("FRAME_TITLE", getTitle()));
    }

}
