/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.framework;


import java.awt.Component;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;

import net.sf.sfac.gui.ExceptionDialog;
import net.sf.sfac.gui.utils.ThinMetalTheme;
import net.sf.sfac.setting.Settings;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Helper class to manage Java L&F action and corresponding setting. <br>
 * In addition to the standard platform defined look-and-feel, this class can also use a metal look-and-feel theme called 'Thin
 * Metal'.
 * 
 * @author Olivier Berlanger
 */
public class LookAndFeelHelper {

    private static Log log = LogFactory.getLog(LookAndFeelHelper.class);

    /** Name of the Metal look-and-feel class (used as base for derived theme names). */
    private static final String METAL_LF_CLASS = "javax.swing.plaf.metal.MetalLookAndFeel";
    /** Key used to store the current look and feel in settings. */
    private static final String LOOK_AND_FEEL_KEY = "frame.lookAndFeel";

    /** The root UI component of the application UI tree (to update UI tree after L&F change). */
    private Component rootUiComponent;
    /** The action managing look-and-feel. */
    private Action[] lookAndFeelActions;
    /** The currently selected look-and-feel. */
    private static String currentLookAndFeel;
    /** The settings used to store the current look-and-feel (null if nothing has to be stored). */
    private Settings setts;


    /**
     * Create a new Look-and-feel helper.
     * 
     * @param sett
     *            The settings used to store the selected look-and-feel (null if nothing has to be stored)
     */
    public LookAndFeelHelper(Settings sett) {
        setts = sett;
        if (setts != null) restoreLookAndFeel();
    }


    /**
     * Set the root Ui Component. It will be used to update the component tree UI once the look-and-feel has changed. <br>
     * As you should create the look-and-feel helper before constructing any UI component (so the correct look-and-feel is selected
     * before the UI is build) you cannot give this component in the constructor.
     * 
     * @param root
     *            The root Ui Component used to update the component tree UI after a look-and-feel change.
     */
    public void setRootUiComponent(Component root) {
        rootUiComponent = root;
    }


    /**
     * Restore the look-and-feel stored in the settings.
     */
    private void restoreLookAndFeel() {
        // get initial look and feel from properties
        String defaultLookAndFeel = UIManager.getCrossPlatformLookAndFeelClassName();
        String startLookAndFeel = setts.getStringProperty(LookAndFeelHelper.LOOK_AND_FEEL_KEY, defaultLookAndFeel);
        try {
            setLookAndFeel(startLookAndFeel);
        } catch (Exception e) {
            ExceptionDialog.showExceptionDialog(null, "Look & Feel error", "Cannot set look & feel:\n" + startLookAndFeel
                    + "\n -> Will use default one.", e);
        }
    }


    /**
     * This method sets the look and feel but don't update the UI.
     * 
     * @param lookAndFeelName
     *            Name of the look-and-feel class to set.
     * @exception Exception
     *                if the look-and-feel cannot be set.
     */
    public static void setLookAndFeel(String lookAndFeelName) throws Exception {
        log.debug("Set look and feel to: " + lookAndFeelName);
        String themeName = null;
        String lookAndFeelClassName;
        int dollarIndex = lookAndFeelName.indexOf('$');
        if (dollarIndex > 0) {
            lookAndFeelClassName = lookAndFeelName.substring(0, dollarIndex);
            themeName = lookAndFeelName.substring(dollarIndex + 1);
        } else {
            lookAndFeelClassName = lookAndFeelName;
        }
        // only one theme is available for the moment
        if (METAL_LF_CLASS.equals(lookAndFeelClassName)) {
            if (themeName != null) MetalLookAndFeel.setCurrentTheme(new ThinMetalTheme());
            else MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme());
        }
        // set look & feel after have setting the right theme
        UIManager.setLookAndFeel(lookAndFeelClassName);
        currentLookAndFeel = lookAndFeelName;
    }


    /**
     * This method sets the look and feel and update the UI. If the look-and-feel cannot be set (for another reason than a missing
     * root UI component), this helper will disable the corresponding action.
     * 
     * @param lookAndFeelName
     *            Name of the look-and-feel class to set.
     * @exception IllegalStateException
     *                if the root UI component has not been set with the <code>setRootUiComponent</code> method.
     */
    public void updateLookAndFeel(String lookAndFeelName) {
        if (rootUiComponent == null) throw new IllegalStateException("No Root Ui Component set");
        try {
            setLookAndFeel(lookAndFeelName);
            SwingUtilities.updateComponentTreeUI(rootUiComponent);
            if (setts != null) setts.setStringProperty(LOOK_AND_FEEL_KEY, lookAndFeelName);
            Action lfAction = getLookAndFeelAction(lookAndFeelName);
            if (lfAction != null) ActionRepository.setActionSelected(lfAction, true);
        } catch (Exception e) {
            log.warn("Look And Feel not available: " + lookAndFeelName, e);
            // disable the action causing the exception (if any)
            Action lfAction = getLookAndFeelAction(lookAndFeelName);
            if (lfAction != null) lfAction.setEnabled(false);
        }
    }


    /**
     * Get the action corresponding to the given look-and-feel class name.
     * 
     * @param lookAndFeelClassName
     *            The look-and-feel class name.
     * @return The action corresponding to the given look-and-feel class name.
     */
    private Action getLookAndFeelAction(String lookAndFeelClassName) {
        int len = lookAndFeelActions.length;
        for (int i = 0; i < len; i++) {
            String actName = (String) lookAndFeelActions[i].getValue(Action.NAME);
            if (actName.equals(lookAndFeelClassName)) return lookAndFeelActions[i];
        }
        return null;
    }


    /**
     * Create a new action for each defined look-and-feel and add it to the ActionRepository. <br>
     * The name of the look and feel will be get from the <code>LookAndFeelInfo</code> of the UIManager (language-independent).
     * <p>
     * Optionally, the menuId+"_menu" can be defined for menu ids defined in the menu path.
     * </p>
     * 
     * @param repo
     *            the repository for all application actions.
     * @param menuName
     *            path to the menu where a menuItem should be added.
     * @param menuGroup
     *            group ids of the menu and menuItems.
     * @param toolbarGroup
     *            group id of the toolbar action.
     * @see ActionRepository#addAction(String, String, String, String, Icon, Action)
     */
    @SuppressWarnings("serial")
    public void createActions(ActionRepository repo, String menuName, String menuGroup, String toolbarGroup) {
        // create Look & Feel actions
        UIManager.LookAndFeelInfo[] infos = UIManager.getInstalledLookAndFeels();
        if (currentLookAndFeel == null) currentLookAndFeel = UIManager.getCrossPlatformLookAndFeelClassName();
        ButtonGroup butGroup = new ButtonGroup();
        int infoLen = infos.length;
        int len = infoLen + 1;
        lookAndFeelActions = new Action[len];
        for (int i = 0; i < len; i++) {
            String className = (i < infoLen) ? infos[i].getClassName() : METAL_LF_CLASS + "$Thin";
            String shortDesc = (i < infoLen) ? infos[i].getName() : "Thin Metal";
            lookAndFeelActions[i] = repo.addAction(className, shortDesc, null, menuName, menuGroup, toolbarGroup, null,
                    new AbstractAction() {

                        public void actionPerformed(ActionEvent ae) {
                            updateLookAndFeel((String) getValue(ActionRepository.ACTION_ID));
                        }
                    });
            ActionRepository.setRadioAction(lookAndFeelActions[i], butGroup);
            ActionRepository.setActionSelected(lookAndFeelActions[i], className.equals(currentLookAndFeel));
        }
        // set an icon on the look_and_feel menu action
        if (menuName != null) {
            int slashIndex = menuName.lastIndexOf('/');
            if (slashIndex >= 0) {
                Action act = repo.getAction(menuName.substring(slashIndex + 1));
                if ((act != null) && (act.getValue(Action.SMALL_ICON) == null)) {
                    ImageIcon icon = SharedResources.getIcon("lookAndFeel.png");
                    act.putValue(Action.SMALL_ICON, icon);
                }

            }
        }
    }

}
