/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.layout;


import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractButton;
import javax.swing.JComponent;
import javax.swing.border.Border;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Action border is a MultiBorder that can either expand/collapse or enable/disable it's content. This border doesn't provide itself
 * the component (button) to control the border state. Programmers can call the setBorderExpanded(..) or setContentEnabled(..) to
 * trigger the action of this border.
 * 
 * @author Olivier Berlanger
 * @see MultiBorderLayout
 * @see MultiBorderConstraints
 */
public class ActionBorder extends MultiBorder implements ActionListener {

    private static Log log = LogFactory.getLog(ActionBorder.class);

    /** Space between the control button component and the border edge. */
    public static final int CONTROL_BUTTON_SPACE = 6;

    public static final int ACTION_COLLAPSE = 0;
    public static final int ACTION_DISABLE = 1;

    private AbstractButton controlButton;
    private int borderAction;
    private Container parentContainer;
    private boolean isExpanded;
    private boolean isContentEnabled;


    /**
     * Creates an ActionBorder using an EtchedBorder and with no button.
     */
    public ActionBorder() {
        isExpanded = true;
        isContentEnabled = true;
    }


    /**
     * Creates an ExpandBorder using an TitledBorder with the given title and with no button.
     * 
     * @param title
     *            the title of the titled border.
     */
    public ActionBorder(String title) {
        super(title);
        isExpanded = true;
        isContentEnabled = true;
    }


    /**
     * Creates an ExpandBorder using the given border and with no button.
     * 
     * @param thePaintedBorder
     *            the border painted by this MultiBorder
     */
    public ActionBorder(Border thePaintedBorder) {
        super(thePaintedBorder);
        isExpanded = true;
        isContentEnabled = true;
    }


    protected void setControlButton(AbstractButton button) {
        controlButton = button;
    }


    protected void setBorderAction(int action) {
        borderAction = action;
    }


    public void setContentEnabled(boolean enabled) {
        if (isContentEnabled != enabled) {
            isContentEnabled = enabled;
            // enable/disable the cells
            int len = enclosedCells.size();
            for (int i = 0; i < len; i++) {
                CellInfo cell = enclosedCells.get(i);
                cell.setEnabled(enabled);
            }
            // enable/disable the inner borders
            len = (nestedBorders == null) ? 0 : nestedBorders.size();
            for (int i = 0; i < len; i++) {
                MultiBorder border = (MultiBorder) nestedBorders.get(i);
                border.setBorderEnabled(enabled);
            }
            // synch the UI
            if (controlButton != null) controlButton.setSelected(enabled);
            if (parentContainer != null) parentContainer.repaint();
        }
    }


    public void setBorderExpanded(boolean expanded) {
        if (isExpanded != expanded) {
            isExpanded = expanded;
            // hide the cells
            int len = enclosedCells.size();
            for (int i = 0; i < len; i++) {
                CellInfo cell = enclosedCells.get(i);
                cell.setCollapsed(isExpanded ? 0 : CellInfo.COLLAPSED_VERTICAL);
            }
            // hide the inner borders
            len = (nestedBorders == null) ? 0 : nestedBorders.size();
            for (int i = 0; i < len; i++) {
                MultiBorder border = (MultiBorder) nestedBorders.get(i);
                border.setBorderVisible(expanded);
            }
            // synch the UI
            if (controlButton != null) controlButton.setSelected(isExpanded);
            relayout();
        }
    }


    private void relayout() {
        if (parentContainer != null) {
            if (parentContainer instanceof JComponent) {
                ((JComponent) parentContainer).revalidate();
            } else {
                parentContainer.invalidate();
            }
            parentContainer.repaint();
        }
    }


    public void setSelected(boolean selected) {
        if (controlButton != null) controlButton.setSelected(selected);
        switch (borderAction) {
            case ACTION_COLLAPSE:
                setBorderExpanded(selected);
                break;
            default:
            case ACTION_DISABLE:
                setContentEnabled(selected);
                break;
        }
    }


    public boolean isSelected() {
        switch (borderAction) {
            case ACTION_COLLAPSE:
                return isExpanded;
            default:
            case ACTION_DISABLE:
                return isContentEnabled;
        }
    }


    // --------------------- Overriden methods of MultiBorder ------------------------------------------

    /**
     * Override to make the nested border invisible when this border is expanded.
     * 
     * @param border
     *            the inner border
     */
    @Override
    protected void setNestedBorder(MultiBorder border) {
        super.setNestedBorder(border);
        border.setBorderVisible(isExpanded);
        border.setBorderEnabled(isContentEnabled);
    }


    /**
     * Override to keep reference of the parent container.
     * 
     * @param cont
     *            The container (usually a JPanel) containing this border and using a MultiBorderLayout.
     */
    @Override
    protected void addExtraComponents(Container cont) {
        if (log.isDebugEnabled()) log.debug("addComponents : Got parentContainer : " + cont.getClass());
        parentContainer = cont;
        super.addExtraComponents(cont);
    }


    /**
     * Override to make the added component invisible when this border is expanded.
     * 
     * @param constr
     *            The constraint used to add the first component to the tabStopRun.
     * @param tabStopRun
     *            the added tabStopRun.
     */
    @Override
    protected void add(CellInfo cell) {
        super.add(cell);
        cell.setCollapsed(isExpanded ? 0 : CellInfo.COLLAPSED_VERTICAL);
        if (log.isDebugEnabled()) log.debug("MultiBorder add cell " + cell + " with content enabled = " + isContentEnabled);
        cell.setEnabled(isContentEnabled);
    }


    @Override
    public void setBorderEnabled(boolean enabled) {
        boolean wasEnabled = isBorderEnabled();
        super.setBorderEnabled(enabled);
        if (wasEnabled != enabled) {
            // when this border is made enabled again, restore it's contentEnabled state
            if (enabled && !isContentEnabled) {
                isContentEnabled = true;
                setContentEnabled(false);
            }
        }
    }


    @Override
    public void setBorderVisible(boolean visible) {
        boolean wasVisible = isBorderVisible();
        super.setBorderVisible(visible);
        if (wasVisible != visible) {
            // when this border is made visible again, restore it's collapsed state
            if (visible && !isExpanded) {
                isExpanded = true;
                setBorderExpanded(false);
            }
        }
    }


    @Override
    protected void layout(Container cont) {
        super.layout(cont);
        if (!isExpanded) {
            Insets inset = getBorderInsets(cont);
            height = inset.top + inset.bottom;
            if (log.isTraceEnabled()) log.trace("Layout collapsed MultiBorder " + x + "," + y + " (" + width + "x" + height + ")");
        }
    }


    /**
     * Add the control button to the list of extra components.
     */
    @Override
    protected void registerExtraComponents() {
        if (controlButton != null) {
            registerExtraComponent(controlButton);
            controlButton.addActionListener(this);
        }
    }


    @Override
    protected void layoutExtraComponents(Container cont) {
        if (controlButton != null) {
            Dimension dim = controlButton.getPreferredSize();
            int posX = x + width - CONTROL_BUTTON_SPACE - dim.width;
            int posY = y + ((getUpdatedInsets(cont).top - dim.height) / 2);
            controlButton.setBounds(posX, posY, dim.width, dim.height);
        }
    }


    @Override
    protected void updateInsets(Insets insetToUpdate) {
        if (controlButton != null) {
            Dimension dim = controlButton.getPreferredSize();
            if (insetToUpdate.top < dim.height) insetToUpdate.top = dim.height;
        }
    }


    // ------------------------ ActionListener interface implementation ------------------------------------------------

    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() == controlButton) {
            boolean selected = controlButton.isSelected();
            switch (borderAction) {
                case ACTION_COLLAPSE:
                    setBorderExpanded(selected);
                    break;
                default:
                case ACTION_DISABLE:
                    setContentEnabled(selected);
                    break;
            }
        }
    }

}
