/*-------------------------------------------------------------------------
 Copyright 2012 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.profiles;


import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JToolBar;

import net.sf.sfac.editor.EditorConfig;
import net.sf.sfac.gui.ExceptionDialog;
import net.sf.sfac.gui.cmp.PartialLineBorder;
import net.sf.sfac.gui.editor.EditorOptionsDialog;
import net.sf.sfac.gui.framework.ActionRepository;
import net.sf.sfac.gui.framework.BarChangeListener;
import net.sf.sfac.gui.framework.SharedResources;
import net.sf.sfac.setting.ProfilesClient;
import net.sf.sfac.setting.SubSettingsList;
import net.sf.sfac.setting.SubSettingsProxy;
import net.sf.sfac.utils.Comparison;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * This controller append a combo-box and 4 buttons in the toolbar to control application profiles. <br>
 * It must be linked to a ProfileClient object providing and using the profiles.
 * 
 * @author Olivier Berlanger
 */
public class ProfilesController implements BarChangeListener {

    private static Log log = LogFactory.getLog(ProfilesController.class);

    private ProfilesClient client;
    private SubSettingsList profiles;

    private JComboBox profilesCombo;
    private Action restoreProfile;
    private Action renameProfile;
    private Action removeProfile;


    public ProfilesController(ProfilesClient theClient, ActionRepository repo) {
        client = theClient;
        profiles = new SubSettingsList(client.getGlobalSettings(), "profiles");
        createActions(repo);
        repo.addBarChangeListener(this);
    }


    @SuppressWarnings("serial")
    private void createActions(ActionRepository repo) {
        // Add profile
        /* storeProfile = */repo.addAction("storeProfile", "Store profile", "Store current profile", "Profile", "d/a1", "z1",
                SharedResources.getIcon("borderAdd.gif"), new AbstractAction() {

                    public void actionPerformed(ActionEvent ae) {
                        storeCurrentSettings();
                    }
                });
        // restore selected profile value
        restoreProfile = repo.addAction("restoreProfile", "Restore profile", "Restore profile values", "Profile", "d/a2", "z2",
                SharedResources.getIcon("refresh.gif"), new AbstractAction() {

                    public void actionPerformed(ActionEvent ae) {
                        refreshProfile();
                    }
                });
        // Rename current profile
        renameProfile = repo.addAction("renameProfile", "Rename profile", "Rename current profile", "Profile", "d/a3", "z3",
                SharedResources.getIcon("rename.gif"), new AbstractAction() {

                    public void actionPerformed(ActionEvent ae) {
                        renameProfile();
                    }
                });
        // Remove current profile
        removeProfile = repo.addAction("removeProfile", "Remove profile", "Remove current profile", "Profile", "d/a4", "z4",
                SharedResources.getIcon("borderRemove.gif"), new AbstractAction() {

                    public void actionPerformed(ActionEvent ae) {
                        removeProfile();
                    }
                });
        checkActionStates();
    }


    public SubSettingsList getProfiles() {
        return profiles;
    }


    void checkActionStates() {
        boolean oneProfileSelected = (profiles.getSelectedItem() != null);
        restoreProfile.setEnabled(oneProfileSelected);
        renameProfile.setEnabled(oneProfileSelected);
        removeProfile.setEnabled(oneProfileSelected);
    }


    private String getSelectedProfileName() {
        SubSettingsProxy current = (SubSettingsProxy) profiles.getSelectedItem();
        return (current == null) ? null : current.getName();
    }


    private SubSettingsProxy getProfile(String profileName) {
        SubSettingsProxy subSett = null;
        int len = profiles.getSize();
        for (int i = 0; i < len; i++) {
            subSett = (SubSettingsProxy) profiles.getSubSettingAt(i);
            if (profileName.equals(subSett.getName())) {
                return subSett;
            }
        }
        return null;
    }


    private void storeCurrentSettings(String profileName) {
        SubSettingsProxy subSet = getProfile(profileName);
        if (subSet == null) {
            log.info("Store new profile with name:" + profileName);
            subSet = (SubSettingsProxy) profiles.addSubSetting();
            subSet.setName(profileName);
            profiles.setSelectedItem(subSet);
        } else {
            log.info("Update profile '" + profileName + "'");
            subSet.clear();
            subSet.setName(profileName);
        }
        subSet.copyValues(client.getCurrentSubSettings());
    }


    private void restoreSelectedSettings() {
        SubSettingsProxy restored = (SubSettingsProxy) profiles.getSelectedItem();
        if (restored != null) {
            client.updateCurrentSubSettings(restored);
        } else {
            log.warn("No profile selected");
        }
    }


    private void changeSelectedProfileName(String oldName, String newName) {
        SubSettingsProxy selectedProfile = (SubSettingsProxy) profiles.getSelectedItem();
        selectedProfile.setName(newName);
        profiles.sort();
    }


    // -------------------------------------------- User actions --------------------------

    void storeCurrentSettings() {
        String name = null;
        try {
            EditorOptionsDialog dial = new EditorOptionsDialog(profilesCombo);
            dial.setTitle("Store Profile");
            StoreProfileParams params = new StoreProfileParams(getSelectedProfileName());
            Object result = dial.showAndEditObject(params);
            if ("OK".equals(result)) {
                name = params.getProfileName();
            }
            if (Comparison.isDefined(name)) {
                log.info("Store current profile with name='" + name + "'");
                storeCurrentSettings(name);
                checkActionStates();
            } else {
                log.info("Store action cancelled by user");
            }
        } catch (Exception e) {
            log.error("Cannot store profile: " + name, e);
            ExceptionDialog.showExceptionDialog(profilesCombo, "Profile error", "Unable to add profile", e);
        }
    }


    void refreshProfile() {
        try {
            restoreSelectedSettings();
        } catch (Exception e) {
            String currentProfileName = getSelectedProfileName();
            log.error("Cannot add refresh pipeline with profile " + currentProfileName, e);
            ExceptionDialog.showExceptionDialog(profilesCombo, "Profile error", "Unable to use current profile: "
                    + currentProfileName, e);
        }
    }


    void renameProfile() {
        String oldName = getSelectedProfileName();
        String newName = null;
        try {
            newName = JOptionPane.showInputDialog(profilesCombo, "New name for current profile");
            if (newName != null) {
                changeSelectedProfileName(oldName, newName);
            } else {
                log.info("Rename action cancelled by user");
            }
        } catch (Exception e) {
            log.error("The current profile (" + oldName + ") cannot be renamed to: " + newName, e);
            ExceptionDialog.showExceptionDialog(profilesCombo, "Profile error", "Unable to rename profile: " + oldName + " --> "
                    + newName, e);
        }
    }


    void removeProfile() {
        String profileName = getSelectedProfileName();
        try {
            SubSettingsProxy selectedProfile = (SubSettingsProxy) profiles.getSelectedItem();
            if (selectedProfile != null) {
                profiles.removeSubSettings(selectedProfile);
                checkActionStates();
            }
        } catch (Exception e) {
            log.error("The current profile (" + profileName + ") cannot be removed", e);
            ExceptionDialog.showExceptionDialog(profilesCombo, "Profile error", "Unable to remove profile: " + profileName, e);
        }
    }

    // ---------------- Little class to hold the 'store profile' action parameters -------------------------------

    public static enum StoreAction {

        OVERRIDE("Override existing profile"), ADD_NEW("Add new profile");

        private String text;


        StoreAction(String txt) {
            text = txt;
        }


        @Override
        public String toString() {
            return text;
        }
    }

    @EditorConfig(label = "")
    public class StoreProfileParams {

        private StoreAction action;

        private String existingProfileName;

        private String newProfileName;


        public StoreProfileParams(String currentProfileName) {
            if (currentProfileName != null) {
                action = StoreAction.OVERRIDE;
                existingProfileName = currentProfileName;
            } else {
                action = StoreAction.ADD_NEW;
            }
        }


        public String getProfileName() {
            return (action == StoreAction.OVERRIDE) ? existingProfileName : newProfileName;
        }


        @EditorConfig(label = "Action", index = 0, editor = "net.sf.sfac.gui.profiles.ProfileNameVisibilityEditor")
        public StoreAction getAction() {
            return action;
        }


        public void setAction(StoreAction act) {
            action = act;
        }


        @EditorConfig(label = "Profile Name", editor = "net.sf.sfac.gui.profiles.ProfilesComboEditor")
        public String getExistingProfileName() {
            ProfilesComboEditor.class.getName();
            return existingProfileName;
        }


        public void setExistingProfileName(String profileName) {
            existingProfileName = profileName;
        }


        @EditorConfig(label = "New Profile Name")
        public String getNewProfileName() {
            return newProfileName;
        }


        public void setNewProfileName(String profileName) {
            newProfileName = profileName;
        }


        public SubSettingsList getProfileList() {
            return getProfiles();
        }

    }


    // --------------------- BarChangeListener interface implementation ------------------------------------------

    public void beforeMnemonicAssignment(ActionRepository repo) {
    }


    public void menubarCreated(ActionRepository repo, JMenuBar menubar) {
    }


    public void toolbarCreated(ActionRepository repo, JToolBar toolbar) {
        int insertIndex = toolbar.getComponentCount() - 4;
        toolbar.setBorder(new PartialLineBorder(new Color(153, 153, 153), PartialLineBorder.BOTTOM, 2));
        toolbar.add(new JLabel("Current Profile:"), insertIndex);
        toolbar.add(Box.createHorizontalStrut(4), insertIndex + 1);
        profilesCombo = new JComboBox(profiles);
        Dimension preferred = profilesCombo.getPreferredSize();
        profilesCombo.setMaximumSize(preferred);
        profilesCombo.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    checkActionStates();
                    refreshProfile();
                }
            }
        });
        toolbar.add(profilesCombo, insertIndex + 2);
        toolbar.add(Box.createHorizontalGlue());
    }

}
