/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.table;


import java.awt.Component;
import java.awt.Graphics;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;


/**
 * Rederer for the table header cells allowing to display a little triangle icon indicating sorted columns.
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class SortHeaderRenderer extends DefaultTableCellRenderer {

    private static Icon sortUpIcon;
    private static Icon sortDownIcon;

    private SortModel sortModel;
    private int columnIndex;
    private int charWidth;

    private Border defaultBorder;
    private Border borderForIcon;
    private Border borderForIconAndNumber;


    public SortHeaderRenderer() {
        if (sortUpIcon == null) loadIcons();
    }


    public SortHeaderRenderer(SortModel srtModel, int colIndex) {
        if (sortUpIcon == null) loadIcons();
        columnIndex = colIndex;
        setSortModel(srtModel);
    }


    public void setColumnIndex(int colIndex) {
        columnIndex = colIndex;
    }


    public void setSortModel(SortModel srtModel) {
        sortModel = srtModel;
        calculateBorder();
    }


    private void loadIcons() {
        try {
            sortUpIcon = new ImageIcon(getClass().getResource("up.png"));
            sortDownIcon = new ImageIcon(getClass().getResource("down.png"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    private void calculateBorder() {
        // get default border
        defaultBorder = UIManager.getBorder("TableHeader.cellBorder");
        // create border including place for just icon
        int leftInset = sortUpIcon.getIconWidth() + 4;
        Border emptyBorder = BorderFactory.createEmptyBorder(0, leftInset, 0, 0);
        borderForIcon = BorderFactory.createCompoundBorder(defaultBorder, emptyBorder);
        // create border including place for icon and number
        charWidth = getFontMetrics(getFont()).charWidth('9');
        leftInset += charWidth;
        emptyBorder = BorderFactory.createEmptyBorder(0, leftInset, 0, 0);
        borderForIconAndNumber = BorderFactory.createCompoundBorder(defaultBorder, emptyBorder);
    }


    /**
     * Returns a table cell renderer for painting table header with sort and filter indicator icons.
     * 
     * @param table
     *            the <code>JTable</code>
     * @param value
     *            the value to assign to the cell at <code>[row, column]</code>
     * @param isSelected
     *            true if cell is selected
     * @param hasFocus
     *            true if cell has focus
     * @param row
     *            the row of the cell to render
     * @param column
     *            the column of the cell to render
     * @return the default table cell renderer
     */
    @Override
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row,
            int column) {
        if (table != null) {
            JTableHeader header = table.getTableHeader();
            if (header != null) {
                setForeground(header.getForeground());
                setBackground(header.getBackground());
            }
        }
        setHorizontalAlignment(JLabel.CENTER);
        setText((value == null) ? "" : value.toString());
        if ((sortModel != null) && sortModel.isColumnSorted(columnIndex)) {
            if (sortModel.getNumberOfSortedColumns() > 1) {
                setBorder(borderForIconAndNumber);
            } else {
                setBorder(borderForIcon);
            }
        } else {
            setBorder(defaultBorder);
        }
        return this;
    }


    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        if ((sortModel != null) && sortModel.isColumnSorted(columnIndex)) {
            Icon sortIcon = (sortModel.getSortDirection(columnIndex) == SortModel.SORT_NORMAL) ? sortUpIcon : sortDownIcon;
            int x = 4;
            if (sortModel.getNumberOfSortedColumns() > 1) {
                String sortIndex = String.valueOf(sortModel.getSortIndex(columnIndex) + 1);
                int height = g.getFontMetrics().getHeight() - 2;
                g.drawString(sortIndex, x, height);
                x += charWidth + 1;
                // } else {
                // x+=charWidth/2 ;
            }
            sortIcon.paintIcon(this, g, x, 1);
        }
    }

}
