/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.table;


import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import net.sf.sfac.setting.Settings;


/**
 * A table that add several functionalities to the standard JTable: sorting, hiding columns, restoring state.
 * <ul>
 * <li>Sorting on multiple columns. You have to call the installSorting() methods to enable it.
 * <li>Selecting the displayed columns. Always enabled.
 * <li>Saving/restoring its state (column size and position, sorting, displayed columns). You have to call the useSettings(..)
 * method to enable it.
 * </ul>
 * The table use a special header where you have a right-click menu to configure sorting and displayed columns. Dragging a column
 * outside the table will hide it. Sort can also be configured by left-click on headers in combination with shift or ctrl keys (see
 * doc).
 * 
 * @author Olivier Berlanger
 */
@SuppressWarnings("serial")
public class SortTable extends JTable {

    private SortModel srtModel;
    private DragOutTableHeader dragTableHeader1;
    private HidingTableColumnModel colModel;


    // ---------------- Reproduce all available constructors from JTable --------------------------------

    /**
     * Constructs a default <code>NewTable</code> that is initialized with a default data model, a default column model, and a
     * default selection model.
     */
    public SortTable() {
        init();
    }


    /**
     * Constructs a <code>NewTable</code> that is initialized with <code>dm</code> as the data model, a default column model, and a
     * default selection model.
     * 
     * @param dm
     *            the data model for the table
     */
    public SortTable(TableModel dm) {
        super(dm);
        init();
    }


    /**
     * Constructs a <code>NewTable</code> that is initialized with <code>dm</code> as the data model, <code>cm</code> as the column
     * model, and a default selection model.
     * 
     * @param dm
     *            the data model for the table
     * @param cm
     *            the column model for the table
     */
    public SortTable(TableModel dm, TableColumnModel cm) {
        super(dm, cm);
        init();
    }


    /**
     * Constructs a <code>NewTable</code> that is initialized with <code>dm</code> as the data model, <code>cm</code> as the column
     * model, and <code>sm</code> as the selection model. If any of the parameters are <code>null</code> this method will initialize
     * the table with the corresponding default model. The <code>autoCreateColumnsFromModel</code> flag is set to false if
     * <code>cm</code> is non-null, otherwise it is set to true and the column model is populated with suitable
     * <code>TableColumns</code> for the columns in <code>dm</code>.
     * 
     * @param dm
     *            the data model for the table
     * @param cm
     *            the column model for the table
     * @param sm
     *            the row selection model for the table
     */
    public SortTable(TableModel dm, TableColumnModel cm, ListSelectionModel sm) {
        super(dm, cm, sm);
        init();
    }


    /**
     * Constructs a <code>NewTable</code> with <code>numRows</code> and <code>numColumns</code> of empty cells using
     * <code>DefaultTableModel</code>. The columns will have names of the form "A", "B", "C", etc.
     * 
     * @param numRows
     *            the number of rows the table holds
     * @param numColumns
     *            the number of columns the table holds
     * @see javax.swing.table.DefaultTableModel
     */
    public SortTable(int numRows, int numColumns) {
        super(numRows, numColumns);
        init();
    }


    /**
     * Constructs a <code>NewTable</code> to display the values in the <code>Vector</code> of <code>Vectors</code>,
     * <code>rowData</code>, with column names, <code>columnNames</code>. The <code>Vectors</code> contained in <code>rowData</code>
     * should contain the values for that row. In other words, the value of the cell at row 1, column 5 can be obtained with the
     * following code:
     * <p>
     * 
     * <pre>
     * ((Vector) rowData.elementAt(1)).elementAt(5);
     * </pre>
     * 
     * <p>
     * Each row must contain a value for each column or an exception will be raised.
     * <p>
     * 
     * @param rowData
     *            the data for the new table
     * @param columnNames
     *            names of each column
     */
    public SortTable(final Vector<?> rowData, final Vector<String> columnNames) {
        super(rowData, columnNames);
        init();
    }


    /**
     * Constructs a <code>NewTable</code> to display the values in the two dimensional array, <code>rowData</code>, with column
     * names, <code>columnNames</code>. <code>rowData</code> is an array of rows, so the value of the cell at row 1, column 5 can be
     * obtained with the following code:
     * <p>
     * 
     * <pre>
     *        rowData[1][5];
     * </pre>
     * 
     * <p>
     * All rows must be of the same length as <code>columnNames</code>.
     * <p>
     * 
     * @param rowData
     *            the data for the new table
     * @param columnNames
     *            names of each column
     */
    public SortTable(final Object[][] rowData, final Object[] columnNames) {
        super(rowData, columnNames);
        init();
    }


    @Override
    public void setModel(TableModel newDataModel) {
        if (srtModel != null) {
            srtModel.setChainedModel(newDataModel);
        } else {
            super.setModel(newDataModel);
        }
    }


    // ---------------- implementation --------------------------------

    private void init() {
        dragTableHeader1 = new DragOutTableHeader();
        setTableHeader(dragTableHeader1);
        dragTableHeader1.addMouseListener(new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent e) {
                mouseClickedInHeader(e);
            }
        });
        colModel = new HidingTableColumnModel();
        setColumnModel(colModel);
    }


    public void useSettings(Settings settngs, String id) {
        TableColumnModel colmnModel = getColumnModel();
        if (colmnModel instanceof HidingTableColumnModel) {
            ((HidingTableColumnModel) colmnModel).useSettings(settngs, id);
        }
        TableModel tblModel = getModel();
        if (tblModel instanceof SortModel) {
            ((SortModel) tblModel).useSettings(settngs, id);
        }
    }


    void mouseClickedInHeader(MouseEvent e) {
        if ((e.getClickCount() == 1) && ((e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0)) {
            JTableHeader tableHdr = (JTableHeader) e.getSource();
            int columnModelIndex = getColumnModelIndex(tableHdr, e);
            if (columnModelIndex >= 0) {
                JPopupMenu popup = new JPopupMenu();
                if (srtModel != null) srtModel.addDefaultPopupMenuItems(popup, tableHdr, e);
                if (colModel != null) colModel.addDefaultPopupMenuItems(popup, tableHdr);
                popup.show(tableHdr, e.getX(), e.getY());
            }
        }
    }


    /**
     * Get the model index of a column clicked in the given table header.
     * 
     * @param tableHdr
     *            the table header
     * @param e
     *            a mouse-click event on the table header
     * @return the model index of a column clicked in the table header.
     */
    private int getColumnModelIndex(JTableHeader tableHdr, MouseEvent e) {
        int columnModelIndex = -1;
        TableColumnModel theColumnModel = tableHdr.getColumnModel();
        int index = theColumnModel.getColumnIndexAtX(e.getX());
        if (index >= 0) {
            TableColumn column = theColumnModel.getColumn(index);
            columnModelIndex = column.getModelIndex();
        }
        return columnModelIndex;
    }


    // ---------------- Sort model management --------------------------------

    public void installSorting() {
        if (srtModel != null) throw new IllegalStateException("Sorting already installed");
        srtModel = new SortModel(getModel());
        super.setModel(srtModel);
        srtModel.registerToTableColumns(this);
    }


    public SortModel getSortModel() {
        return srtModel;
    }


    public int getSelectedRowInRootModel() {
        int selectedRow = getSelectedRow();
        if (selectedRow >= getRowCount()) selectedRow = -1;
        if ((srtModel != null) && (selectedRow >= 0)) {
            selectedRow = srtModel.getChainedModelRow(selectedRow);
        }
        return selectedRow;
    }


    public void setSelectedRowInRootModel(int chainedModelRow) {
        if ((chainedModelRow >= 0) && (chainedModelRow < getRowCount())) {
            int rowToSelect = chainedModelRow;
            if (srtModel != null) {
                rowToSelect = srtModel.getRow(chainedModelRow);
            }
            setRowSelectionInterval(rowToSelect, rowToSelect);
        } else {
            clearSelection();
        }
    }


    public int[] getSelectedRowsInRootModel() {
        int[] selectedRows = getSelectedRows();
        if (srtModel != null) {
            int len = selectedRows.length;
            for (int i = 0; i < len; i++) {
                selectedRows[i] = srtModel.getChainedModelRow(selectedRows[i]);
            }
        }
        return selectedRows;
    }

}
