/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.table;


import java.awt.Component;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Class setting the size of a table columns so that they can display all their content. The <code>JTable</code> AutoResizeMode is
 * set to <code>JTable.AUTO_RESIZE_OFF</code> when <code>startColumnSizing()</code> is called, as this class takes the column sizing
 * responsability.
 * <p>
 * The sizing process is independent from the paint process of the table, so you can also size the table before showing it with this
 * utility class.
 * <p>
 * As column sizing requires to check the size of each cell, this class might run as a background task for large tables. This class
 * check the first 100 rows synchonously (when <code>startColumnSizing()</code> is called) which takes approx 10 to 20 millis. Then
 * if there is more rows, a background thread is started to check the size of the rest of the rows (by batch of 100). Column size
 * are continously adapted by this thread (until it find the largest row).
 * <p>
 * All the <code>startColumnSizing()</code> calls sent to this class must be done using the event-dispatch thread.
 * 
 * @author Olivier Berlanger
 */
public class TableColumnSizer implements Runnable {

    private static final Log log = LogFactory.getLog(TableColumnSizer.class);

    private static final int ROW_BATCH_SIZE = 100;
    private static final int SLEEP_TIME = 0;
    private static final int DEFAULT_PADDING = 10;

    private JTable table;
    private int currentRow;
    private int totalPreferredWidth;
    private int[] columnPreferredWidths;
    private boolean continueSizeCheck;
    private Thread checkingThread;

    private int defaultColumnPadding = DEFAULT_PADDING;
    private int[] columnPaddings;


    public TableColumnSizer(JTable theTable) {
        table = theTable;
    }


    public void setDefaultColumnPadding(int inset) {
        defaultColumnPadding = inset;
    }


    public int getDefaultColumnPadding() {
        return defaultColumnPadding;
    }


    public void setColumnPaddings(int[] insets) {
        columnPaddings = insets;
    }


    public int[] getColumnPaddings() {
        return columnPaddings.clone();
    }


    public void startColumnSizing() {
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        totalPreferredWidth = 0;
        int nbrColumns = table.getColumnCount();
        columnPreferredWidths = new int[nbrColumns];
        currentRow = 0;
        continueSizeCheck = true;
        checkRowBatch();
        if (continueSizeCheck && (checkingThread == null)) {
            checkingThread = new Thread(this, "TableSizer");
            checkingThread.start();
        }
    }


    private int getPaddingForColumn(int col) {
        if ((columnPaddings != null) && (col < columnPaddings.length)) return columnPaddings[col];
        return defaultColumnPadding;
    }


    private void checkHeaderSize() {
        JTableHeader tableHeader = table.getTableHeader();
        TableColumnModel tcm = table.getColumnModel();
        TableColumn column;
        TableCellRenderer renderer;
        Component comp;
        int preferredWidth;
        int nbrColumns = table.getColumnCount();
        int cellInset = table.getRowMargin();
        for (int i = 0; i < nbrColumns; i++) {
            column = tcm.getColumn(i);
            renderer = column.getHeaderRenderer();
            if (renderer == null) renderer = tableHeader.getDefaultRenderer();
            comp = renderer.getTableCellRendererComponent(table, column.getHeaderValue(), false, false, -1, i);
            preferredWidth = comp.getPreferredSize().width + cellInset + getPaddingForColumn(i);
            if (columnPreferredWidths[i] < preferredWidth) {
                totalPreferredWidth += preferredWidth - columnPreferredWidths[i];
                columnPreferredWidths[i] = preferredWidth;
            }
        }
    }


    void checkRowBatch() {
        // long startTime=System.currentTimeMillis() ;
        TableCellRenderer renderer;
        Component comp;
        int preferredWidth;
        int j;
        int nbrColumns = table.getColumnCount();
        int nbrRows = table.getRowCount();
        int previousTotal = totalPreferredWidth;
        int cellInset = table.getRowMargin() * 2;
        int endRow = currentRow + ROW_BATCH_SIZE;
        if (endRow > nbrRows) endRow = nbrRows;
        if (currentRow < endRow) {
            if (currentRow == 0) checkHeaderSize();
            for (int i = currentRow; i < endRow; i++) {
                for (j = 0; j < nbrColumns; j++) {
                    renderer = table.getCellRenderer(i, j);
                    comp = table.prepareRenderer(renderer, i, j);
                    preferredWidth = comp.getPreferredSize().width + cellInset + getPaddingForColumn(j);
                    if (columnPreferredWidths[j] < preferredWidth) {
                        totalPreferredWidth += preferredWidth - columnPreferredWidths[j];
                        columnPreferredWidths[j] = preferredWidth;
                    }
                }
            }
        }
        // System.out.print("Batch "+currentRow+" -> "+endRow+" total size="+totalPreferredWidth+" sizes=[") ;
        // for (j=0 ; j<nbrColumns ; j++) System.out.print(" "+columnPreferredWidths[j]) ;
        // System.out.println("]") ;
        if (previousTotal != totalPreferredWidth) resizeTableColumns();
        currentRow = endRow;
        if (currentRow >= nbrRows) {
            continueSizeCheck = false;
            if (checkingThread != null) {
                checkingThread = null;
                throw new NullPointerException("Kill checking thread");
            }
        }
        // System.out.println("Batch of row sized in "+(System.currentTimeMillis()-startTime)+" millis") ;
    }


    private void resizeTableColumns() {
        // System.out.print("**** Resize table to "+totalPreferredWidth+" sizes=[") ;
        TableColumnModel tcm = table.getColumnModel();
        int nbrColumns = tcm.getColumnCount();
        for (int i = 0; i < nbrColumns; i++) {
            // System.out.print(" "+columnPreferredWidths[i]) ;
            TableColumn column = tcm.getColumn(i);
            // column.setResizable(false) ;
            column.setPreferredWidth(columnPreferredWidths[i]);
            column.setWidth(columnPreferredWidths[i]);
        }
        // System.out.println("]") ;
        table.repaint();
    }


    @SuppressWarnings("unused")
    public void run() {
        try {
            while (continueSizeCheck) {
                if (SLEEP_TIME > 0) Thread.sleep(SLEEP_TIME);
                SwingUtilities.invokeAndWait(new Runnable() {

                    public void run() {
                        checkRowBatch();
                    }
                });
            }
        } catch (InterruptedException ie) {
            log.error("Size checking thread interrupted", ie);
        } catch (InvocationTargetException ite) {
            log.warn("End of size checking thread: " + Thread.currentThread() + " on exception", ite);
        }
    }

}
