/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.utils;


import java.awt.Component;
import java.awt.Container;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetContext;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;

import net.sf.sfac.file.FileType;
import net.sf.sfac.file.FileUtils;
import net.sf.sfac.gui.SystemDialog;
import net.sf.sfac.utils.Comparison;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Object listening for File drag-and-drop. <br>
 * This object can register itself on a tree of component (a component and all its sub-components) with the
 * <code>registerToComponent(...)</code> method. It handles the plumbing of drag-&-drop and delegates the real action (handling the
 * dropped file(s)) to an object implementing the FileDropTarget interface.
 */
public class FileDropListener implements DropTargetListener {

    private static Log log = LogFactory.getLog(FileDropListener.class);
    private static final String[] OPTIONS_FOR_DROP = { "OVERRIDE", "RENAME", "CANCEL" };

    public static final DataFlavor URI_LIST_FLAVOR = new DataFlavor(String.class, "text/uri-list");

    private FileDropTarget target;
    private int acceptableActions;
    private int currentAction;
    private List<FileType> acceptedFiles;
    private boolean isAccepted = false;


    /**
     * Create a file-drop listener.
     * 
     * @param fileDropTarget
     *            the object that will actually handle the dropped files.
     * @param acceptedFileType
     *            An object able to filter the dropped files. If null, all files are accepted.
     */
    public FileDropListener(FileDropTarget fileDropTarget, FileType... acceptedFileType) {
        this(fileDropTarget, DnDConstants.ACTION_COPY_OR_MOVE, acceptedFileType);
    }


    /**
     * Create a file-drop listener.
     * 
     * @param fileDropTarget
     *            the object that will actually handle the dropped files.
     * @param acceptedActions
     *            the accepted drag-&-drop actions (specified by DnDConstants: usually DnDConstants.ACTION_COPY_OR_MOVE).
     * @param acceptedFileType
     *            An object able to filter the dropped files. If null, all files are accepted.
     */
    public FileDropListener(FileDropTarget fileDropTarget, int acceptedActions, FileType... acceptedFileType) {
        target = fileDropTarget;
        acceptedFiles = new ArrayList<FileType>();
        if (acceptedFileType != null) {
            for (FileType type : acceptedFileType) {
                acceptedFiles.add(type);
            }
        }
        acceptableActions = acceptedActions;
    }


    public List<FileType> getAcceptedFileType() {
        return acceptedFiles;
    }


    public void setAcceptedFileType(List<FileType> fileTypes) {
        if (fileTypes == null) throw new IllegalArgumentException("fileTypes cannot be null");
        acceptedFiles = fileTypes;
    }


    public void addAcceptedFileType(FileType addedType) {
        acceptedFiles.add(addedType);
    }


    public void registerToComponent(Component cmp) {
        cmp.setDropTarget(new DropTarget(cmp, this));
        if (cmp instanceof Container) {
            Container cnt = (Container) cmp;
            int len = cnt.getComponentCount();
            for (int i = 0; i < len; i++)
                registerToComponent(cnt.getComponent(i));
        }
    }


    /**
     * Utility method to really execute the drop action once all the source/target files are resolved. <br>
     * This method never erase an existing file without warning the user.
     * 
     * @param dropAction
     *            the drop action: DnDConstants.ACTION_MOVE or DnDConstants.ACTION_COPY
     * @param sourceFile
     *            the source (dropped file)
     * @param targetFile
     *            the target file
     * @param existingFile
     *            an existing file that will be replaced by the target (but possibly with another extension)
     * @param parent
     *            the parent component to be able to display modal dialog
     * @throws IOException
     *             if something failed.
     */
    public static void executeDropAction(int dropAction, File sourceFile, File targetFile, File existingFile, Component parent)
            throws IOException {
        boolean continueAction = true;
        // check existing image
        if (existingFile != null) {
            // prompt dialog before erasing existing image
            Object[] params = new Object[] { existingFile.getAbsolutePath() };
            int res = SystemDialog.showDialog(parent, "DROP_OVERRIDE", "DROP_OVERRIDE_FILE", params, OPTIONS_FOR_DROP, "RENAME",
                    JOptionPane.WARNING_MESSAGE);
            if ((res == JOptionPane.CLOSED_OPTION) || (res == 1)) {
                // rename (default option)
                if (log.isDebugEnabled()) log.debug("Rename existing image file: " + existingFile);
                FileUtils.renameToBackup(existingFile);
            } else if (res == 0) {
                // replace (do a delete to avoid the same name with two different extensions)
                if (log.isDebugEnabled()) log.debug("Delete existing image file: " + existingFile);
                FileUtils.deleteFile(existingFile);
            } else {
                // cancel
                if (log.isDebugEnabled()) log.debug("Image drop cancelled by user");
                continueAction = false;
            }
        } else {
            FileUtils.ensureParentDirectoryExists(targetFile);
        }
        // move/copy the image
        if (continueAction) {
            if (dropAction == DnDConstants.ACTION_MOVE) {
                // move-rename the file
                FileUtils.moveFile(sourceFile, targetFile);
            } else if (dropAction == DnDConstants.ACTION_COPY) {
                // copy the file
                FileUtils.copyFile(sourceFile, targetFile);
            } else {
                throw new IllegalArgumentException("Unknown drop action: " + dropAction);
            }
        }
    }

    // ---------------------- Definition of a listener interface --------------------------------------------

    public static interface FileDropTarget {

        public void filesDropped(File[] files, int dropAction);

    }


    // ---------------------- Implementation of DropTargetListener interface ---------------------------------

    /**
     * Called when a drag operation has encountered the <code>DropTarget</code>.
     * 
     * @param dtde
     *            the <code>DropTargetDragEvent</code>
     */
    public void dragEnter(DropTargetDragEvent dtde) {
        currentAction = dtde.getDropAction();
        boolean flavorSupported = dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor)
                || dtde.isDataFlavorSupported(URI_LIST_FLAVOR);
        isAccepted = flavorSupported && ((currentAction & acceptableActions) != 0);
        if (log.isDebugEnabled()) log.debug("dragEnter: action = " + currentAction + " --> accepted = " + isAccepted);
        // List<DataFlavor> flavors=dtde.getCurrentDataFlavorsAsList();
        // for (DataFlavor flavor : flavors) {
        // try {
        // System.out.println("     * "+flavor);
        // System.out.println("           --> "+dtde.getTransferable().getTransferData(flavor));
        // } catch(Exception ioe) {
        // System.out.println("        !!! ERROR: "+ioe );
        // }
        // }
        if (isAccepted) dtde.acceptDrag(currentAction);
        else dtde.rejectDrag();
    }


    /**
     * Called when a drag operation is ongoing on the <code>DropTarget</code>.
     * 
     * @param dtde
     *            the <code>DropTargetDragEvent</code>
     */
    public void dragOver(DropTargetDragEvent dtde) {
        if (isAccepted) dtde.acceptDrag(currentAction);
        else dtde.rejectDrag();
    }


    /**
     * Called if the user has modified the current drop gesture.
     * 
     * @param dtde
     *            the <code>DropTargetDragEvent</code>
     */
    public void dropActionChanged(DropTargetDragEvent dtde) {
        dragEnter(dtde);
    }


    /**
     * The drag operation has departed the <code>DropTarget</code> without dropping.
     * 
     * @param dte
     *            the <code>DropTargetEvent</code>
     */
    public void dragExit(DropTargetEvent dte) {
        currentAction = DnDConstants.ACTION_NONE;
        isAccepted = false;
    }


    /**
     * The drag operation has terminated with a drop on this <code>DropTarget</code>. <br>
     * This method is responsible for undertaking the transfer of the data associated with the gesture. The
     * <code>DropTargetDropEvent</code> provides a means to obtain a <code>Transferable</code> object that represents the data
     * object(s) to be transfered.
     * <P>
     * From this method, the <code>DropTargetListener</code> shall accept or reject the drop via the acceptDrop(int dropAction) or
     * rejectDrop() methods of the <code>DropTargetDropEvent</code> parameter.
     * </P>
     * <P>
     * Subsequent to acceptDrop(), but not before, <code>DropTargetDropEvent</code>'s getTransferable() method may be invoked, and
     * data transfer may be performed via the returned <code>Transferable</code>'s getTransferData() method.
     * </P>
     * <P>
     * At the completion of a drop, an implementation of this method is required to signal the success/failure of the drop by
     * passing an appropriate <code>boolean</code> to the <code>DropTargetDropEvent</code>'s dropComplete(boolean success) method.
     * </P>
     * <P>
     * Note: The actual processing of the data transfer is not required to finish before this method returns. It may be deferred
     * until later.
     * </P>
     * 
     * @param dtde
     *            the <code>DropTargetDropEvent</code>
     */
    @SuppressWarnings("unchecked")
    public void drop(DropTargetDropEvent dtde) {
        DropTargetContext targetContext = dtde.getDropTargetContext();
        List<File> droppedFiles = null;

        // check if current action is available
        if ((dtde.getSourceActions() & currentAction) != 0) {
            // Check for the file[] flavor
            if (dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
                dtde.acceptDrop(currentAction);
                try {
                    // get the dropped data
                    Transferable t = dtde.getTransferable();
                    droppedFiles = (List<File>) t.getTransferData(DataFlavor.javaFileListFlavor);
                } catch (Exception e) {
                    log.error("Cannot convert dropped object to File[]", e);
                }
            } else if (dtde.isDataFlavorSupported(URI_LIST_FLAVOR)) {
                dtde.acceptDrop(currentAction);
                String uriList = "?";
                try {
                    // get the dropped data
                    Transferable t = dtde.getTransferable();
                    uriList = (String) t.getTransferData(URI_LIST_FLAVOR);
                    String[] uris = uriList.split("\n");
                    droppedFiles = new ArrayList<File>();
                    for (String uri : uris) {
                        File f = new File(new URI(uri.trim()).getPath());
                        droppedFiles.add(f);
                    }
                } catch (Exception e) {
                    log.error("Cannot convert dropped URI list to file: " + uriList, e);
                }
            } else {
                log.warn("Flavor 'javaFileListFlavor' not available in DropTargetDropEvent");
                dtde.rejectDrop();
            }
        } else {
            log.warn("Current action (" + currentAction + ") not available in DropTargetDropEvent source actions ("
                    + dtde.getSourceActions() + ")");
            dtde.rejectDrop();
        }

        if (droppedFiles != null) {
            if (log.isDebugEnabled()) log.debug("Notify dropped file list: " + droppedFiles);
            targetContext.dropComplete(true);
            File[] fileArray = getFilteredFiles(droppedFiles);
            if (fileArray.length > 0) {
                target.filesDropped(fileArray, currentAction);
            } else {
                log.warn("Filtered file lis is empty (for " + droppedFiles.size() + " files dropped)");
            }
        } else {
            targetContext.dropComplete(false);
        }
        // re-init drag status
        currentAction = DnDConstants.ACTION_NONE;
        isAccepted = false;
    }


    /**
     * Filter the dropped files with the specified file filter.
     */
    private File[] getFilteredFiles(List<File> droppedFiles) {
        List<File> filteredFiles;
        if (Comparison.isEmpty(acceptedFiles)) {
            filteredFiles = droppedFiles;
        } else {
            filteredFiles = new ArrayList<File>();
            for (File dropped : droppedFiles) {
                if (isAccepted(dropped)) {
                    filteredFiles.add(dropped);
                }
            }
        }
        return filteredFiles.toArray(new File[filteredFiles.size()]);
    }


    private boolean isAccepted(File fil) {
        boolean accepted = false;
        for (FileType type : acceptedFiles) {
            if (type.accept(fil)) {
                accepted = true;
                break;
            }
        }
        if (log.isDebugEnabled()) log.debug("   - Dropped file: " + fil + " --> accepted = " + accepted);
        return accepted;
    }

}
