/*-------------------------------------------------------------------------
 Copyright 2009 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.sfac.gui.utils;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.List;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;


public class TreeHelper {

    /** Comparator user to sort the tree path from longest to shortest. */
    private static final PathSorter PATH_SORTER = new PathSorter();


    /**
     * Expand all nodes of a JTree with level less than the given one. So all the nodes with the same level as the given one should
     * be visible.
     * 
     * @param tree
     *            The tree owning the nodes to expand.
     * @param pathToLevel
     *            The path giving the reference node (giving the level).
     */
    public static void expandAllNodesToLevel(JTree tree, TreePath pathToLevel) {
        int level = (pathToLevel == null) ? 0 : pathToLevel.getPathCount();
        if (level > 1) {
            TreePath currentPath = new TreePath(tree.getModel().getRoot());
            expandNodesOfLevels(tree, currentPath, level);
        }
    }


    /**
     * Expand all the child nodes of the given tree node.
     * 
     * @param tree
     *            The tree owning the nodes to expand.
     * @param pathToNone
     *            The path to root node of the subtree to expand.
     */
    public static void expandChildNodes(JTree tree, TreePath pathToNone) {
        expandNodesOfLevels(tree, pathToNone, Integer.MAX_VALUE);
    }


    /**
     * Expand all node of a tree.
     * 
     * @param tree
     *            The tree owning the nodes to expand.
     */
    public static void expandAll(JTree tree) {
        TreePath rootPath = new TreePath(tree.getModel().getRoot());
        expandNodesOfLevels(tree, rootPath, Integer.MAX_VALUE);
    }


    /**
     * Collapse all node with level greater or equals to the given one. So all the nodes with the same level as the given one (which
     * were previously visible) should be visible but no more.
     * 
     * @param tree
     *            The tree owning the nodes to collapse.
     * @param pathToLevel
     *            The path giving the reference node (giving the level).
     */
    public static void collapseAllNodesFromLevel(JTree tree, TreePath pathToLevel) {
        int level = (pathToLevel == null) ? -1 : pathToLevel.getPathCount() - 1;
        if (level >= 1) {
            TreePath rootPath = new TreePath(tree.getModel().getRoot());
            collapseNodesOfLevels(tree, rootPath, level);
        }
    }


    /**
     * Collapse all the child nodes of the given tree node.
     * 
     * @param tree
     *            The tree owning the nodes to collapse.
     * @param pathToNone
     *            The path to root node of the subtree to collapse.
     */
    public static void collapseChildNodes(JTree tree, TreePath pathToNone) {
        collapseNodesOfLevels(tree, pathToNone, 1);
    }


    /**
     * Collapse all nodes.
     * 
     * @param tree
     *            The tree owning the nodes to collapse.
     */
    public static void collapseAll(JTree tree) {
        TreePath rootPath = new TreePath(tree.getModel().getRoot());
        collapseNodesOfLevels(tree, rootPath, 1);
    }


    private static void expandNodesOfLevels(JTree tree, TreePath currentPath, int toLevel) {
        int currentLevel = currentPath.getPathCount();
        if (currentLevel < toLevel) {
            if (tree.isCollapsed(currentPath)) {
                tree.expandPath(currentPath);
                collapseChildNodes(tree, currentPath);
                tree.expandPath(currentPath);
            }
            Object currentNode = currentPath.getLastPathComponent();
            TreeModel model = tree.getModel();
            RecursiveTreePath childPath;
            int len = model.getChildCount(currentNode);
            for (int i = 0; i < len; i++) {
                childPath = new RecursiveTreePath(currentPath, model.getChild(currentNode, i));
                expandNodesOfLevels(tree, childPath, toLevel);
            }
        }
    }


    private static void collapseNodesOfLevels(JTree tree, TreePath basePath, int fromLevel) {
        TreePath path;
        int level;
        Enumeration<TreePath> expandedEnum = tree.getExpandedDescendants(basePath);
        if (expandedEnum != null) {
            List<TreePath> expandedList = new ArrayList<TreePath>();
            for (; expandedEnum.hasMoreElements();)
                expandedList.add(expandedEnum.nextElement());
            Collections.sort(expandedList, PATH_SORTER);
            int len = expandedList.size();
            for (int i = 0; i < len; i++) {
                path = expandedList.get(i);
                level = path.getPathCount();
                if (level > fromLevel) {
                    tree.collapsePath(path);
                }
            }
        }
    }

    static class PathSorter implements Comparator<TreePath> {

        public int compare(TreePath path1, TreePath path2) {
            if (path1 == path2) return 0;
            if (path1 == null) return -1;
            if (path2 == null) return 1;
            int len1 = path1.getPathCount();
            int len2 = path2.getPathCount();
            if (len1 != len2) return (len1 < len2) ? 1 : -1;
            return 0;
        }
    }


    public static void registerHelperListener(final JTree tree) {
        tree.addMouseListener(new MouseAdapter() {

            @Override
            public void mousePressed(MouseEvent mouseEvt) {
                if (mouseEvt.getButton() == MouseEvent.BUTTON3) {
                    final TreePath path = tree.getPathForLocation(mouseEvt.getX(), mouseEvt.getY());
                    if (path != null) {
                        tree.setSelectionPath(path);
                        JPopupMenu popup = new JPopupMenu();
                        JMenuItem expandAllItem = new JMenuItem("Expand All");
                        expandAllItem.addActionListener(new ActionListener() {

                            public void actionPerformed(ActionEvent e) {
                                expandAll(tree);
                            }
                        });
                        popup.add(expandAllItem);
                        JMenuItem expandChildItem = new JMenuItem("Expand Children");
                        expandChildItem.addActionListener(new ActionListener() {

                            public void actionPerformed(ActionEvent e) {
                                expandChildNodes(tree, path);
                            }
                        });
                        popup.add(expandChildItem);
                        JMenuItem expandLevelItem = new JMenuItem("Expand To This Level");
                        expandLevelItem.addActionListener(new ActionListener() {

                            public void actionPerformed(ActionEvent e) {
                                expandAllNodesToLevel(tree, path);
                            }
                        });
                        popup.add(expandLevelItem);
                        JMenuItem collapseAllItem = new JMenuItem("Collapse All");
                        collapseAllItem.addActionListener(new ActionListener() {

                            public void actionPerformed(ActionEvent e) {
                                collapseAll(tree);
                            }
                        });
                        popup.add(collapseAllItem);
                        JMenuItem collapseChildItem = new JMenuItem("Collapse Children");
                        collapseChildItem.addActionListener(new ActionListener() {

                            public void actionPerformed(ActionEvent e) {
                                collapseChildNodes(tree, path);
                            }
                        });
                        popup.add(collapseChildItem);
                        JMenuItem collapseLevelItem = new JMenuItem("Collapse To This Level");
                        collapseLevelItem.addActionListener(new ActionListener() {

                            public void actionPerformed(ActionEvent e) {
                                collapseAllNodesFromLevel(tree, path);
                            }
                        });
                        popup.add(collapseLevelItem);
                        popup.show(tree, mouseEvt.getX(), mouseEvt.getY());
                    }
                }
            }
        });
    }

    // ---------- Override TreePath just to increase visiblity of needed constructor ------------------

    @SuppressWarnings("serial")
    public static class RecursiveTreePath extends TreePath {

        public RecursiveTreePath(TreePath parent, Object node) {
            super(parent, node);
        }
    }

}
